#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the DDSim WorkflowModule."""

from __future__ import print_function
from __future__ import absolute_import

import unittest
import os
import shutil
import tempfile
import tarfile
from zipfile import ZipFile
from mock import patch, MagicMock as Mock, mock_open
import six

from DIRAC import gLogger, S_OK, S_ERROR
from ILCDIRAC.Workflow.Modules.DDSimAnalysis import DDSimAnalysis
from Tests.Utilities.GeneralUtils import assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Workflow.Modules.DDSimAnalysis'
MODULEBASE_NAME = 'ILCDIRAC.Workflow.Modules.ModuleBase'
PROXYINFO_NAME = 'DIRAC.Core.Security.ProxyInfo'
DD4H_NAME = 'ILCDIRAC.Workflow.Utilities.DD4hepMixin'
BUILTIN_NAME = 'builtins' if six.PY3 else '__builtin__'

# pylint: disable=too-many-public-methods, protected-access

gLogger.setLevel("ERROR")
gLogger.showHeaders(True)


def cleanup(tempdir):
  """Remove files after run."""
  try:
    shutil.rmtree(tempdir)
  except OSError:
    pass


@patch("%s.getProxyInfoAsString" % MODULEBASE_NAME, new=Mock(return_value=S_OK()))
@patch("%s.getProxyInfoAsString" % PROXYINFO_NAME, new=Mock(return_value=S_OK()))
class TestDDSimAnalysis(unittest.TestCase):
  """test DDSimAnalysis."""

  def assertIn(self, *args, **kwargs):
    """make this existing to placate pylint."""
    return super(TestDDSimAnalysis, self).assertIn(*args, **kwargs)

  @patch("%s.getProxyInfoAsString" % MODULEBASE_NAME, new=Mock(return_value=S_OK()))
  @patch("%s.getProxyInfoAsString" % PROXYINFO_NAME, new=Mock(return_value=S_OK()))
  def setUp(self):
    self.ddsim = DDSimAnalysis()
    self.curdir = os.getcwd()
    self.tempdir = tempfile.mkdtemp("", dir="./")
    os.chdir(self.tempdir)
    self.ddsim.ops = Mock()

  def tearDown(self):
    os.chdir(self.curdir)
    cleanup(self.tempdir)


class TestDDSimAnalysisParticleTable(TestDDSimAnalysis):
  """test DDSim createParticleTable."""

  def setUp(self):
    """Help with setup."""
    super(TestDDSimAnalysisParticleTable, self).setUp()
    self.logFileName = "python101.log"
    with open(self.logFileName, "w") as logF:
      logF.write("logged the logging logs")

  @patch('%s.open' % BUILTIN_NAME, new_callable=mock_open, read_data="export DD4hepINSTALL=\"some/path\"")
  def test_DDSim_particleTable_success(self, mo):
    """DDSim.createParticleTable...................................................................."""
    self.ddsim.extraParticles = [[25, "Maribor", 2, 1500., 10., 100.]]
    handlers = (mo.return_value, mock_open(read_data=" 2 u 2 0.33000 0.00000 0.00000E+00").return_value)
    mo.side_effect = handlers
    with patch("os.path.exists", new=Mock(side_effect=[True])):
      res = self.ddsim.createParticleTable("Dummy/path", ['SIM.physics.pdgfile = None', 'SIM.physics.list = "AList"'])
    print(res)
    self.assertIn("25 Maribor 2 1500.0 10.0 100.0", res['Value'])

  @patch('%s.open' % BUILTIN_NAME, new_callable=mock_open, read_data="export DD4hepINSTALL=\"some/path\"")
  def test_DDSim_particleTable_success2(self, mo):
    """DDSim.createParticleTable...................................................................."""
    self.ddsim.extraParticles = [[13, "mu", 2, 1500., 10., 100.]]
    handlers = (mo.return_value, mock_open(read_data=" 2 u 2 0.33000 0.00000 0.00000E+00\n"
                                                     "  13 mu^- -3 0.10566 0.00000 6.58654E+05").return_value)
    mo.side_effect = handlers
    with patch("os.path.exists", new=Mock(side_effect=[True])):
      res = self.ddsim.createParticleTable("Dummy/path", [])
    print(res)
    self.assertEqual([' 2 u 2 0.33000 0.00000 0.00000E+00', '13 mu 2 1500.0 10.0 100.0'], res['Value'])

  @patch('%s.open' % BUILTIN_NAME, new_callable=mock_open, read_data=" 2 u 2 0.33000 0.00000 0.00000E+00\n")
  def test_DDSim_particleTable_success3(self, mo):
    """DDSim.createParticleTable...................................................................."""
    self.ddsim.extraParticles = [[13, "mu", 2, 1500., 10., 100.]]
    with patch("os.path.exists", new=Mock(side_effect=[True])):
      res = self.ddsim.createParticleTable("Dummy/path", ['SIM.physics.pdgfile = "/here/is/the/file"'])
      print(res)
      self.assertEqual([' 2 u 2 0.33000 0.00000 0.00000E+00', '13 mu 2 1500.0 10.0 100.0'], res['Value'])

  @patch("%s.open" % BUILTIN_NAME, mock_open(read_data="export INSTAL=\"some/path\""))
  def test_DDSim_particleTable_fail(self):
    """DDSim.createParticleTable...................................................................."""
    self.ddsim.extraParticles = [[25, "Maribor", 2., 1500., 10., 100.]]
    res = self.ddsim.createParticleTable("Dummy/path", [])
    print(res)
    self.assertIn("Did not find env", res['Message'])

  @patch('%s.open' % BUILTIN_NAME, new_callable=mock_open, read_data=" 2 u 2 0.33000 0.00000 0.00000E+00\n")
  def test_DDSim_particleTable_fail2(self, mo):
    """DDSim.createParticleTable...................................................................."""
    self.ddsim.extraParticles = [[13, "mu", 2, 1500., 10., 100.]]
    res = self.ddsim.createParticleTable("Dummy/path", ['SIM.physics.pdgfile = "/here/is/the/file"'])
    print(res)
    self.assertIn("/here/is/the/file", res['Message'])

  @patch('%s.open' % BUILTIN_NAME, new_callable=mock_open, read_data="read OK")
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_resolveDDSimOptions_success(self, mo):
    """DDSim.resolveDDSimOption...................................................................."""
    with patch("os.chmod", new=Mock(side_effect=[True])):
      res = self.ddsim.resolveDDSimOptions("/path/to/end", "CLIC_o2_something")
    print(res)
    self.assertIn("read OK", res['Value'])

  @patch('%s.open' % BUILTIN_NAME, new_callable=mock_open, read_data="")
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_resolveDDSimOptions_fail(self, mo):
    """DDSim.resolveDDSimOption...................................................................."""
    with patch("os.chmod", new=Mock(side_effect=[True])):
      res = self.ddsim.resolveDDSimOptions("/path/to/end", "CLIC_o2_something")
    print(res)
    self.assertIn("empty", res['Message'])


class TestDDSimAnalysisRunit(TestDDSimAnalysis):
  """test DDSim runtIt."""

  def setUp(self):
    super(TestDDSimAnalysisRunit, self).setUp()
    self.logFileName = "python101.log"
    with open(self.logFileName, "w") as logF:
      logF.write("logged the logging logs")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_runIt_success(self):
    """DDSim.runit ................................................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    # side effect for Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, True])):
      res = self.ddsim.runIt()
    print(res)
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_OK(["pairs.hepmc"])))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_runIt_success_inputFile(self):
    """DDSim.runit success with inputFile..........................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.InputFile = "pairs.hepmc"
    # side effect for Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    self.assertEqual(self.ddsim.InputFile, ["pairs.hepmc"])
    self.assertIn(" --inputFile pairs.hepmc ", self.ddsim.extraCLIarguments)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_ERROR("no pairs.hepmc")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_runIt_failure_inputFile(self):
    """DDSim.runit failure with inputFile..........................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.InputFile = "pairs.hepmc"
    # side effect for Script, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, False, True])):
      res = self.ddsim.runIt()
    self.assertEqual(res['Message'], "no pairs.hepmc")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_OK("pairs.hepmc")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_runIt_failure_LogFile(self):
    """DDSim.runit failure with applicationLog......................................................"""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.InputFile = "pairs.hepmc"
    self.ddsim.ignoreapperrors = False
    # side effect for Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False])):
      res = self.ddsim.runIt()
    self.assertIn("did not produce the expected log", res['Message'])

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_OK("pairs.hepmc")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_runIt_failure_LogFile_ignore(self):
    """DDSim.runit failure with applicationLog but ignore..........................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.InputFile = "pairs.hepmc"
    self.ddsim.ignoreapperrors = True
    # side effect for Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.resolveIFpaths" % MODULE_NAME, new=Mock(return_value=S_OK("pairs.hepmc")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_runIt_success_LogAndScriptPresent(self):
    """DDSim.runit success log and script exist....................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.InputFile = "pairs.hepmc"
    self.ddsim.ignoreapperrors = True
    with open("DDSim__Run_.sh", "w") as scr:
      scr.write("content")
    with open(self.logFileName, "w") as scr:
      scr.write("content")
    # side effect for Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[True, False, True, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  def test_DDSim_runIt_success_steeringFile(self):
    """DDSim.runit success with steeringFile........................................................"""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    # side effect for Steering1, Steering2, Script, userlib, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[True, True, False, False, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    self.assertEqual(self.ddsim.SteeringFile, "mySteering.py")
    steerFlag = " --steeringFile %s " % self.ddsim.SteeringFile
    self.assertIn(steerFlag, self.ddsim.extraCLIarguments)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK("SteerFold")))
  def test_DDSim_runIt_success_steeringFile_1(self):
    """DDSim.runit success with non-local steeringFile.............................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    # side effect for Steering1a, Steering1b, Steering2, Script, userlib, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, True, False, False, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    fullPath = os.path.join("SteerFold", "mySteering.py")
    self.assertEqual(self.ddsim.SteeringFile, fullPath)
    steerFlag = " --steeringFile %s " % fullPath
    self.assertIn(steerFlag, self.ddsim.extraCLIarguments)

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_ERROR("NothingToSee")))
  def test_DDSim_runIt_failure_steeringFile_1(self):
    """DDSim.runit failure with non-local steeringFile.............................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    # side effect for Steering1a, Steering1b, Steering2, Script, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, True, False, False, True])):
      res = self.ddsim.runIt()
    self.assertFalse(res['OK'])
    self.assertEqual(res['Message'], "NothingToSee")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK("SteerFold")))
  def test_DDSim_runIt_failure_steeringFile_2(self):
    """DDSim.runit failure with non-local steeringFile 2............................................"""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    # side effect for Steering1a, Steering1b, Steering2, Script, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, False, False, False, False, True])):
      res = self.ddsim.runIt()
    self.assertFalse(res['OK'])
    self.assertEqual(res['Message'], "Could not find steering file")

  #######################
  # Test NumberOfEvents #
  #######################

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK("SteerFold")))
  def test_DDSim_runIt_success_numberOfEvents_1(self):
    """DDSim.runit success with NumberOfEvents set.................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    self.ddsim.NumberOfEvents = 123
    # side effect for Steering1a, Steering1b, Steering2, Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, True, False, False, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    self.assertIn(" --numberOfEvents 123 ", self.ddsim.extraCLIarguments)

  ##############################
  # Test skipNEvents/startFrom #
  ##############################
  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK("SteerFold")))
  def test_DDSim_runIt_success_skipNevents(self):
    """DDSim.runit success with startFrom..........................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    self.ddsim.NumberOfEvents = 123
    self.ddsim.startFrom = 22
    # side effect for Steering1a, Steering1b, Steering2, Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, True, False, False, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    self.assertIn(" --skipNEvents 22 ", self.ddsim.extraCLIarguments)

  ##############
  # Test Debug #
  ##############
  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK("SteerFold")))
  def test_DDSim_runIt_success_Debug(self):
    """DDSim.runit success with startFrom..........................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    self.ddsim.NumberOfEvents = 123
    self.ddsim.debug = True
    # side effect for Steering1a, Steering1b, Steering2, Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, True, False, False, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    self.assertIn(" --printLevel DEBUG ", self.ddsim.extraCLIarguments)

  ########################
  # Test extra particles #
  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK("SteerFold")))
  @patch("%s.DDSimAnalysis.createParticleTable" % MODULE_NAME, new=Mock(return_value=S_OK(["test"])))
  @patch("%s.DDSimAnalysis.resolveDDSimOptions" % MODULE_NAME, new=Mock(return_value=S_OK(["test"])))
  def test_DDSim_runIt_success_extraParticles(self):
    """DDSim.runit success with extra particles....................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering2.py"
    self.ddsim.NumberOfEvents = 123
    self.ddsim.extraParticles = [[25, "Maribor", 2., 1500., 10., 100.]]
    # side effect for Steering1a, Steering1b, Steering2, Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, True, False, False, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    self.assertIn("--physics.pdgfile particle.tbl", self.ddsim.extraCLIarguments)

  ###################
  # Test OutputFile #
  ###################
  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_OK("myDet.xml")))
  @patch("%s.shellCall" % MODULE_NAME, new=Mock(return_value=S_OK((0, "AllGood"))))
  @patch("%s.getSteeringFileDirName" % MODULE_NAME, new=Mock(return_value=S_OK("SteerFold")))
  def test_DDSim_runIt_success_OutputFile_1(self):
    """DDSim.runit success with OutputFile set......................................................"""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.SteeringFile = "mySteering.py"
    self.ddsim.OutputFile = "grailDiary.root"
    # side effect for Steering1a, Steering1b, Steering2, Script, userlibs, log, logAfter
    with patch("os.path.exists", new=Mock(side_effect=[False, True, True, False, False, False, True])):
      res = self.ddsim.runIt()
    assertDiracSucceeds(res, self)
    self.assertIn(" --outputFile grailDiary.root ", self.ddsim.extraCLIarguments)

  def test_DDSim_runIt_fail_1(self):
    """DDSim.runit failure platform................................................................."""
    self.ddsim.applicationLog = self.logFileName
    res = self.ddsim.runIt()
    self.assertEqual(res['Message'], 'No ILC platform selected')

  def test_DDSim_runIt_fail_2(self):
    """DDSim.runit failure log......................................................................"""
    self.ddsim.platform = "Windows"
    res = self.ddsim.runIt()
    self.assertEqual(res['Message'], 'No Log file provided')

  def test_DDSim_runIt_fail_3(self):
    """DDSim.runit failure neither.................................................................."""
    res = self.ddsim.runIt()
    self.assertEqual(res['Message'], 'No ILC platform selected')

  def test_DDSim_runIt_fail_4(self):
    """DDSim.runit fail steps......................................................................."""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    self.ddsim.workflowStatus = S_ERROR("Failed earlier")
    res = self.ddsim.runIt()
    self.assertEqual(res['Value'], "DDSim should not proceed as previous step did not end properly")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_ERROR("missing ddsiming.sh")))
  def test_DDSim_runIt_fail_env(self):
    """DDSim.runit failed to get env................................................................"""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    res = self.ddsim.runIt()
    self.assertEqual(res['Message'], "missing ddsiming.sh")

  @patch("%s.getEnvironmentScript" % MODULE_NAME, new=Mock(return_value=S_OK("ddsiming.sh")))
  @patch("%s.DDSimAnalysis._getDetectorXML" % MODULE_NAME, new=Mock(return_value=S_ERROR("no myDet.xml")))
  def test_DDSim_runIt_fail_xml(self):
    """DDSim.runit failed to get xml................................................................"""
    self.ddsim.platform = "Windows"
    self.ddsim.applicationLog = self.logFileName
    res = self.ddsim.runIt()
    self.assertEqual(res['Message'], "no myDet.xml")


class TestDDSimAnalysisEnv(TestDDSimAnalysis):
  """DDSim getEnvScript."""

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getEnvScript_success(self):
    """DDSim.getEnvScript success..................................................................."""
    platform = "Windows"
    appname = "ddsim"
    appversion = "Vista"
    res = self.ddsim.getEnvScript(platform, appname, appversion)
    self.assertEqual(res['Value'], os.path.abspath("DDSimEnv.sh"))
    self.assertTrue(os.path.exists(os.path.abspath("DDSimEnv.sh")))
    with open("DDSimEnv.sh") as script:
      scriptLines = "".join(script.readlines())
      self.assertIn("declare -x DD4hepINSTALL=%s" % "/win32", scriptLines)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_ERROR("no softFolder")))
  def test_DDSim_getEnvScript_noSoftFolder(self):
    """DDSim.getEnvScript fail softfolder..........................................................."""
    platform = "Windows"
    appname = "ddsim"
    appversion = "Vista"
    res = self.ddsim.getEnvScript(platform, appname, appversion)
    self.assertFalse(res['OK'])
    self.assertEqual(res['Message'], "no softFolder")

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  @patch("os.path.exists", new=Mock(return_value=True))
  def test_DDSim_getEnvScript_vars(self):
    """DDSim.getEnvScript with variables success...................................................."""

    platform = "Windows"
    appname = "ddsim"
    appversion = "Vista"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(KNIGHTSWORD="Ni",
                                                                    WHEN="Always",
                                                                    G4LEDATA="/dev/camelot",
                                                                   )
                                                             )
                                        )
    res = self.ddsim.getEnvScript(platform, appname, appversion)
    self.assertEqual(res['Value'], os.path.abspath("DDSimEnv.sh"))
    self.assertTrue(os.path.exists(os.path.abspath("DDSimEnv.sh")))
    with open("DDSimEnv.sh") as script:
      scriptLines = "".join(script.readlines())
      self.assertIn("declare -x KNIGHTSWORD=Ni", scriptLines)
      self.assertIn("declare -x WHEN=Always", scriptLines)
      self.assertIn("declare -x G4LEDATA=/dev/camelot", scriptLines)
      self.assertNotIn("declare -x G4LEDATA=$(ls -d $G4DATA/", scriptLines)
      self.assertIn("declare -x G4LEVELGAMMADATA=$(ls -d $G4DATA/", scriptLines)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  @patch("os.path.exists", new=Mock(return_value=True))
  def test_DDSim_getEnvScript_noVars(self):
    """DDSim.getEnvScript with no additional variables.............................................."""

    platform = "Windows"
    appname = "ddsim"
    appversion = "Vista"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_ERROR("No Variables Set"))
    self.ddsim.ops.getValue = Mock(return_value=None)
    res = self.ddsim.getEnvScript(platform, appname, appversion)
    self.ddsim.ops.getValue.assert_called_once_with("/AvailableTarBalls/Windows/ddsim/Vista/InitScript", None)
    self.assertEqual(res['Value'], os.path.abspath("DDSimEnv.sh"))
    self.assertTrue(os.path.exists(os.path.abspath("DDSimEnv.sh")))
    with open("DDSimEnv.sh") as script:
      scriptLines = "".join(script.readlines())
      self.assertIn("declare -x G4LEDATA=$(ls -d $G4DATA/", scriptLines)
      self.assertNotIn("source ", scriptLines)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  @patch("%s.getNewLDLibs" % MODULE_NAME, new=Mock(return_value=""))
  def test_DDSim_getEnvScript_init(self):
    """DDSim.getEnvScript with initscript..........................................................."""
    platform = "Windows"
    appname = "ddsim"
    appversion = "Vista"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(KNIGHTSWORK="Ni",
                                                                    WHEN="Always"
                                                                   )
                                                             )
                                        )
    self.ddsim.ops.getValue = Mock(return_value="/cvmfs/initthisfile.sh")
    with patch("os.path.exists", new=Mock(return_value=True)):
      res = self.ddsim.getEnvScript(platform, appname, appversion)
    self.ddsim.ops.getValue.assert_called_once_with("/AvailableTarBalls/Windows/ddsim/Vista/InitScript", None)
    self.assertEqual(res['Value'], os.path.abspath("DDSimEnv.sh"))
    self.assertTrue(os.path.exists(os.path.abspath("DDSimEnv.sh")))
    with open("DDSimEnv.sh") as script:
      scriptLines = "".join(script.readlines())
      self.assertIn("source /cvmfs/initthisfile.sh", scriptLines)


class TestDDSimAnalysisASI(TestDDSimAnalysis):
  """DDSim.ApplicationSpecificInputs."""

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DDSim_ASI_NoVariables(self):
    """DDSim.applicationSpecificInputs: checks that no variables have been set after this call......"""
    gLogger.setLevel("ERROR")
    self.ddsim.workflow_commons = dict()
    self.ddsim.applicationSpecificInputs()
    self.assertFalse(self.ddsim.jobReport or self.ddsim.productionID)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DDSim_ASI_StartFrom(self):
    """DDSim.applicationSpecificInputs: check setting of startfrom variable........................."""
    gLogger.setLevel("ERROR")
    self.ddsim.workflow_commons = dict(StartFrom=321)
    self.ddsim.resolveInputVariables()
    self.ddsim.applicationSpecificInputs()
    self.assertEqual(self.ddsim.startFrom, 321)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DDSim_ASI_RandomSeed_Prod(self):
    """DDSim.applicationSpecificInputs: check setting of randomseed in production..................."""
    gLogger.setLevel("ERROR")
    self.ddsim.workflow_commons = dict(StartFrom=321, IS_PROD=True, PRODUCTION_ID=6666, JOB_ID=123)
    self.ddsim.OutputFile = 'simulation.root'
    self.ddsim.resolveInputVariables()
    self.ddsim.applicationSpecificInputs()
    self.assertEqual(self.ddsim.randomSeed, 6666123)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DDSim_ASI_RandomSeed_Set(self):
    """DDSim.applicationSpecificInputs: check setting of default randomseed in user jobs............"""
    gLogger.setLevel("ERROR")
    self.ddsim = DDSimAnalysis()
    self.ddsim.workflow_commons = dict()
    self.ddsim.resolveInputVariables()
    self.ddsim.applicationSpecificInputs()
    self.assertEqual(int(self.ddsim.randomSeed), 12345)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DDSim_ASI_RandomSeed_User(self):
    """DDSim.applicationSpecificInputs: check setting of randomseed in user jobs...................."""
    gLogger.setLevel("ERROR")
    self.ddsim = DDSimAnalysis()
    self.ddsim.randomSeed = 654321
    self.ddsim.workflow_commons = dict()
    self.ddsim.resolveInputVariables()
    self.ddsim.applicationSpecificInputs()
    self.assertEqual(int(self.ddsim.randomSeed), 654321)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DDSim_ASI_RandomSeed_User_Zero(self):
    """DDSim.applicationSpecificInputs: check setting of randomseed to zero in user jobs............"""
    gLogger.setLevel("ERROR")
    self.ddsim = DDSimAnalysis()
    self.ddsim.randomSeed = 0
    self.ddsim.workflow_commons = dict()
    self.ddsim.resolveInputVariables()
    self.ddsim.applicationSpecificInputs()
    self.assertEqual(int(self.ddsim.randomSeed), 0)

  @patch.dict(os.environ, {"JOBID": "12345"})
  def test_DDSim_ASI_InputFiles(self):
    """DDSim.applicationSpecificInputs: check setting inputData....................................."""
    gLogger.setLevel("ERROR")
    self.ddsim = DDSimAnalysis()
    self.ddsim.InputData = ["myslcio.slcio", "mystdhep.HEPEvt", "my.notforsimulation"]
    self.ddsim.workflow_commons = dict()
    self.ddsim.resolveInputVariables()
    self.ddsim.applicationSpecificInputs()
    self.assertEqual(self.ddsim.InputFile, ["myslcio.slcio", "mystdhep.HEPEvt"])


class TestDDSimAnalysisDetXMLCS(TestDDSimAnalysis):
  """tests for _getDetectorXML."""

  @patch("%s.getSoftwareFolder" % DD4H_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML(self):
    """DDSim.getDetectorXML from CS................................................................."""
    gLogger.setLevel("ERROR")
    xmlPath = "/path/to/camelot.xml"
    self.ddsim.detectorModel = "camelot"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot=xmlPath)))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._getDetectorXML()
    self.assertEqual(res['Value'], xmlPath)

  @patch("%s.getSoftwareFolder" % DD4H_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_relPatg(self):
    """DDSim.getDetectorXML from CS with relative path.............................................."""
    gLogger.setLevel("ERROR")
    xmlPath = "rel/path/to/camelot.xml"
    self.ddsim.detectorModel = "camelot"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot=xmlPath)))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._getDetectorXML()
    self.assertEqual(res['Value'], os.path.join("/win32", xmlPath))

  @patch("%s.getSoftwareFolder" % DD4H_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_Fail(self):
    """DDSim.getDetectorXML Failure................................................................."""
    gLogger.setLevel("ERROR")
    xmlPath = "/path/to/camelot.xml"
    self.ddsim.detectorModel = "fortressOfSolitude"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot=xmlPath)))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._getDetectorXML()
    self.assertFalse(res['OK'])
    self.assertEqual(res['Message'], "Detector model was not found")

  @patch("%s.getSoftwareFolder" % DD4H_NAME, new=Mock(return_value=S_ERROR("Windows not supported")))
  def test_DDSim_getDetectorXML_NoSoftFolder(self):
    """DDSim.getDetectorXML Error no SoftwareFolder................................................."""
    gLogger.setLevel("ERROR")
    xmlPath = "/path/to/camelot.xml"
    self.ddsim.detectorModel = "camelot"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot=xmlPath)))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._getDetectorXML()
    self.assertEqual(res['Message'], "Windows not supported")

  @patch("%s.getSoftwareFolder" % DD4H_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_NoDetModels(self):
    """DDSim.getDetectorXML Error no detectorModels................................................."""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "camelot"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_ERROR("Nothing to see"))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._getDetectorXML()
    self.assertEqual(res['Message'], "Failed to get list of DetectorModels from the ConfigSystem")

  @patch("os.path.exists", new=Mock(return_value=True))
  @patch("%s.unzip_file_into_dir" % DD4H_NAME, new=Mock())
  def test_DDSim_getDetectorXML_CustomWithOfficialName(self):
    """DDSim.getDetectorXML CustomTarball with official name........................................"""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "Camelot"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK({"Camelot": "/path/to/camelot.xml"}))
    self.ddsim.workflow_commons = dict()
    # only works for python 2
    with patch('%s.open' % BUILTIN_NAME, mock_open(read_data="RoundTable")):
      res = self.ddsim._getDetectorXML()
    print(res)
    self.assertEqual(res['Value'], os.path.join("Camelot", "Camelot.xml"))


class TestDDSimAnalysisDetXMLTar(TestDDSimAnalysis):
  """tests for _getDetectorXML."""

  def setUp(self):
    super(TestDDSimAnalysisDetXMLTar, self).setUp()
    self.ddsim.detectorModel = "myDet"
    outputFilename = self.ddsim.detectorModel + ".tar.gz"
    os.makedirs(self.ddsim.detectorModel)
    xmlPath = os.path.join(self.ddsim.detectorModel, self.ddsim.detectorModel + ".xml")
    with open(xmlPath, "w") as xml:
      xml.write("myDet is awesome")
    with tarfile.open(outputFilename, "w:gz") as tar:
      tar.add(xmlPath)
    cleanup(self.ddsim.detectorModel)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_Local_TarGZ(self):
    """DDSim.getDetectorXML with local tar.gz......................................................."""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "myDet"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot="/dev/null")))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._getDetectorXML()
    gLogger.error(" res ", res)
    expectedPath = os.path.join(os.getcwd(), self.ddsim.detectorModel, self.ddsim.detectorModel + ".xml")
    self.assertEqual(res['Value'], expectedPath)
    self.assertTrue(os.path.exists(expectedPath))

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_Local_TarGZ_2(self):
    """DDSim.getDetectorXML with local tar.gz run twice............................................."""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "myDet"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot="/dev/null")))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._extractTar()
    res = self.ddsim._extractTar()
    gLogger.error(" res ", res)
    expectedPath = os.path.join(os.getcwd(), self.ddsim.detectorModel, self.ddsim.detectorModel + ".xml")
    self.assertEqual(res['Value'], expectedPath)
    self.assertTrue(os.path.exists(expectedPath))

  def test_DDSim_extractTar_Raise(self):
    """DDSim._extractTar raised exception..........................................................."""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "myDet"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot="/dev/null")))
    self.ddsim.workflow_commons = dict()
    with patch("tarfile.open", side_effect=RuntimeError("This is what happens")):
      res = self.ddsim._extractTar()
    self.assertFalse(res['OK'])
    self.assertEqual(res['Message'], "Failed to untar detector model")

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_Local_TGZ_2(self):
    """DDSim.getDetectorXML with local tgz.........................................................."""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "myDet"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot="/dev/null")))
    self.ddsim.workflow_commons = dict()
    os.rename("myDet.tar.gz", "myDet.tgz")
    res = self.ddsim._getDetectorXML()
    gLogger.error(" res ", res)
    expectedPath = os.path.join(os.getcwd(), self.ddsim.detectorModel, self.ddsim.detectorModel + ".xml")
    self.assertEqual(res['Value'], expectedPath)
    self.assertTrue(os.path.exists(expectedPath))


class TestDDSimAnalysisDetXMLZip(TestDDSimAnalysis):
  """tests for _getDetectorXML when a zip file exists."""

  def setUp(self):
    super(TestDDSimAnalysisDetXMLZip, self).setUp()
    self.ddsim.detectorModel = "myDet"
    outputFilename = self.ddsim.detectorModel + ".zip"
    os.makedirs(self.ddsim.detectorModel)
    xmlPath = os.path.join(self.ddsim.detectorModel, self.ddsim.detectorModel + ".xml")
    with open(xmlPath, "w") as xml:
      xml.write("myDet is awesome")
    with ZipFile(outputFilename, "w") as zipF:
      zipF.write(xmlPath)
    cleanup(self.ddsim.detectorModel)

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_Local_Zip(self):
    """DDSim.getDetectorXML with local zip.........................................................."""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "myDet"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot="/dev/null")))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._getDetectorXML()
    gLogger.error(" res ", res)
    expectedPath = os.path.join(os.getcwd(), self.ddsim.detectorModel, self.ddsim.detectorModel + ".xml")
    self.assertEqual(res['Value'], expectedPath)
    self.assertTrue(os.path.exists(expectedPath))

  @patch("%s.getSoftwareFolder" % MODULE_NAME, new=Mock(return_value=S_OK("/win32")))
  def test_DDSim_getDetectorXML_Local_Zip_2(self):
    """DDSim.getDetectorXML with local zip run twice................................................"""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "myDet"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot="/dev/null")))
    self.ddsim.workflow_commons = dict()
    res = self.ddsim._extractZip()
    res = self.ddsim._extractZip()
    gLogger.error(" res ", res)
    expectedPath = os.path.join(os.getcwd(), self.ddsim.detectorModel, self.ddsim.detectorModel + ".xml")
    self.assertEqual(res['Value'], expectedPath)
    self.assertTrue(os.path.exists(expectedPath))

  def test_DDSim_extractZip_Raise(self):
    """DDSim._extractZip raised exception..........................................................."""
    gLogger.setLevel("ERROR")
    self.ddsim.detectorModel = "myDet"
    self.ddsim.ops.getOptionsDict = Mock(return_value=S_OK(dict(camelot="/dev/null")))
    self.ddsim.workflow_commons = dict()
    # myDet.zip does not exist
    os.remove("myDet.zip")
    res = self.ddsim._extractZip()
    self.assertFalse(res['OK'])
    self.assertEqual(res['Message'], "Failed to unzip detector model")


def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysis)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysisParticleTable))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysisRunit))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysisDetXMLTar))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysisDetXMLZip))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysisDetXMLCS))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysisEnv))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(TestDDSimAnalysisASI))

  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Create and send a combined request for any pending operations at the end of a job.

:author: S. Poss
:author: S. Paterson
"""

from __future__ import absolute_import
import os

from DIRAC.TransformationSystem.Client.FileReport import FileReport
from DIRAC import S_OK, S_ERROR, gLogger

from ILCDIRAC.Workflow.Modules.ModuleBase import ModuleBase

__RCSID__ = '$Id$'
LOG = gLogger.getSubLogger(__name__)


class FailoverRequest(ModuleBase):
  """Handle the failover requests issued by previous steps.

  Used in production.
  """
  #############################################################################

  def __init__(self):
    """Module initialization."""
    super(FailoverRequest, self).__init__()
    self.version = __RCSID__
    # Internal parameters
    self.enable = True
    self.jobID = ''
    self.productionID = None
    self.prodJobID = None
    # Workflow parameters
    self.jobReport = None
    self.fileReport = None
    self.jobType = ''

  #############################################################################
  def applicationSpecificInputs(self):
    """By convention the module input parameters are resolved here."""
    if 'JOBID' in os.environ:
      self.jobID = os.environ['JOBID']
      LOG.verbose('Found WMS JobID = %s' % self.jobID)
    else:
      LOG.info('No WMS JobID found, disabling module via control flag')
      self.enable = False

    self.enable = self.step_commons.get('Enable', self.enable)
    if not isinstance(self.enable, bool):
      LOG.error('Enable flag set to non-boolean value %s, setting to False' % self.enable)
      self.enable = False

    # Earlier modules will have populated the report objects
    self.jobReport = self.workflow_commons.get('JobReport', self.jobReport)

    self.fileReport = self.workflow_commons.get('FileReport', self.fileReport)

    self.productionID = self.workflow_commons.get('PRODUCTION_ID', self.productionID)

    self.prodJobID = self.workflow_commons.get('JOB_ID', self.prodJobID)

    return S_OK('Parameters resolved')

  #############################################################################
  def execute(self):
    """Main execution function."""
    LOG.info('Initializing %s' % self.version)
    result = self.resolveInputVariables()
    if not result['OK']:
      LOG.error("Failed to resolve input parameters:", result['Message'])
      return result

    if not self.enable:
      LOG.info('Module is disabled by control flag')
      return S_OK('Module is disabled by control flag')

    self.fileReport = self.fileReport if self.fileReport else FileReport('Transformation/TransformationManager')

    if self.InputData:
      inputFiles = self.fileReport.getFiles()
      for lfn in self.InputData:
        if lfn not in inputFiles:
          LOG.verbose('No status populated for input data %s, setting to "Unused"' % lfn)
          self.fileReport.setFileStatus(int(self.productionID), lfn, 'Unused')

    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.info('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
      inputFiles = self.fileReport.getFiles()
      for lfn in inputFiles:
        if inputFiles[lfn] != 'ApplicationCrash' and self.jobType != "Split":
          LOG.info('Forcing status to "Unused" due to workflow failure for: %s' % (lfn))
          self.fileReport.setFileStatus(int(self.productionID), lfn, 'Unused')
    else:
      inputFiles = self.fileReport.getFiles()
      if inputFiles:
        LOG.info('Workflow status OK, setting input file status to Processed')
      for lfn in inputFiles:
        LOG.info('Setting status to "Processed" for: %s' % (lfn))
        self.fileReport.setFileStatus(int(self.productionID), lfn, 'Processed')

    fileReportCommitResult = self.fileReport.commit()
    if fileReportCommitResult['OK']:
      LOG.info('Status of files have been properly updated in the ProcessingDB')
    else:
      LOG.error('Failed to report file status to ProductionDB:', fileReportCommitResult['Message'])
      LOG.error('Request will be generated.')
      disetResult = self.fileReport.generateForwardDISET()
      if not disetResult['OK']:
        LOG.warn("Could not generate Operation for file report with result:\n%s" % disetResult['Message'])
      else:
        if disetResult['Value'] is None:
          LOG.info("Files correctly reported to TransformationDB")
        else:
          request = self._getRequestContainer()
          request.addOperation(disetResult['Value'])
          self.workflow_commons['Request'] = request

    # Must ensure that the local job report instance is used to report the final status
    # in case of failure and a subsequent failover operation
    if self.workflowStatus['OK'] and self.stepStatus['OK']:
      self.jobReport.setApplicationStatus('Job Finished Successfully')

    resFF = self.generateFailoverFile()
    if not resFF['OK']:
      LOG.error(resFF['Message'])

    return self.finalize()

  #############################################################################
  def finalize(self):
    """Finalize and report correct status for the workflow based on the workflow or step status."""
    LOG.verbose('Workflow status = %s, step status = %s' % (self.workflowStatus['OK'], self.stepStatus['OK']))
    if not self.workflowStatus['OK'] or not self.stepStatus['OK']:
      LOG.warn('Workflow status is not ok, will not overwrite status')
      LOG.info('Workflow failed, end of FailoverRequest module execution.')
      return S_ERROR('Workflow failed, FailoverRequest module completed')

    LOG.info('Workflow successful, end of FailoverRequest module execution.')
    return S_OK('FailoverRequest module completed')

#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#



#!/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Prepare the production summary tables.

Options:
   -P, --prods prodID            Productions: greater than with gt1234, range with 32-56, list with 34,56
   -p, --precise_detail          Precise detail, slow
   -t, --types prodTypeList      Production Types, comma separated, default all
   -S, --Statuses statusList     Statuses, comma separated, default all
   -N, --sample_size             The number of generator files to sample for cross-section and "luminosity, default 100
"""
from __future__ import absolute_import
from six.moves import range
__RCSID__ = "$Id$"

from collections import defaultdict
import os
import random
import textwrap

from DIRAC.Core.Base import Script
from DIRAC.Core.Utilities.DIRACScript import DIRACScript
from DIRAC.Core.Utilities.List import breakListIntoChunks
from DIRAC import S_OK, exit as dexit, S_ERROR


def _getFileInfo(lfn, fc):
  """Retrieve the file info."""
  from DIRAC.Core.Utilities import DEncode
  from DIRAC import gLogger
  lumi = 0.0
  nbevts = 0
  res = fc.getFileUserMetadata(lfn)
  if not res['OK']:
    gLogger.error("Failed to get metadata of %s" % lfn)
    return (0, 0, {})

  lumi += float(res['Value'].get('Luminosity', 0.0))
  nbevts += int(res['Value'].get('NumberOfEvents', 0))

  addinfo = {}
  if 'AdditionalInfo' in res['Value']:
    addinfo = res['Value']['AdditionalInfo']
    if addinfo.count("{"):
      addinfo = eval(addinfo)
    else:
      addinfo = DEncode.decode(addinfo)[0]
  xsec = addinfo.get('xsection', {}).get('sum', {}).get('xsection', 0.0)
  return (float(lumi), int(nbevts), float(xsec))


def _translate(detail):
  """Replace whizard naming convention by human conventions."""
  detail = detail.replace('v', 'n1:n2:n3:N1:N2:N3')
  detail = detail.replace('qli', 'u:d:s:U:D:S')
  detail = detail.replace('ql', 'u:d:s:c:b:U:D:S:C:B')
  detail = detail.replace('q', 'u:d:s:c:b:t')
  detail = detail.replace('Q', 'U:D:S:C:B:T')
  detail = detail.replace('e1', 'e-')
  detail = detail.replace('E1', 'e+')
  detail = detail.replace('e2', 'mu-')
  detail = detail.replace('E2', 'mu+')
  detail = detail.replace('e3', 'tau-')
  detail = detail.replace('E3', 'tau+')
  detail = detail.replace('n1', 'nue')
  detail = detail.replace('N1', 'nueb')
  detail = detail.replace('n2', 'numu')
  detail = detail.replace('N2', 'numub')
  detail = detail.replace('n3', 'nutau')
  detail = detail.replace('N3', 'nutaub')
  detail = detail.replace('U', 'ubar')
  detail = detail.replace('C', 'cbar')
  detail = detail.replace('T', 'tbar')
  detail = detail.replace('tbareV', 'TeV')
  detail = detail.replace('D', 'dbar')
  detail = detail.replace('S', 'sbar')
  detail = detail.replace('B', 'bbar')
  detail = detail.replace('Z0', 'Z')
  detail = detail.replace('Z', 'Z0')
  detail = detail.replace('gghad', 'gamma gamma -> hadrons')
  detail = detail.replace(',', '')
  detail = detail.replace('n N', 'nu nub')
  detail = detail.replace('se--', 'seL-')
  detail = detail.replace('se-+', 'seL+')
  detail = detail.replace(' -> ', '->')
  detail = detail.replace('->', ' -> ')
  detail = detail.replace(' H ->', ', H ->')
  detail = detail.replace(' Z0 ->', ', Z0 ->')
  detail = detail.replace(' W ->', ', W ->')

  return detail

# def getAncestor(lfn):
#  from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
#  fc = FileCatalogClient()
#  res = fc.getFileAncestors([lfn],1)
#  if not res['OK']:
#    return S_ERROR("Failed getting ancestor")
#  for ancestor in res['Value']['Successful'][lfn].keys():
#    if not ancestor.count("stdhep"):
#      res = getAncestor(ancestor)
#      if not res['OK']:
#        return S_ERROR("Failed geting ancestor")
#    else:
#      return S_OK(ancestor)


class _Params(object):
  """CLI Parameters class."""

  def __init__(self):
    """Initialize."""
    self.prod = []
    self.minprod = 0
    self.full_det = False
    self.sampleSize = 100
    self.ptypes = ['MCGeneration', 'MCSimulation', 'MCReconstruction', "MCReconstruction_Overlay"]
    self.statuses = ['Active', 'Stopped', 'Completed', 'Archived']

  def setProdID(self, opt):
    """Set the prodID to use.

    Can be a range, a list, a unique value and a 'greater than' value.
    """
    if opt.count(","):
      parts = opt.split(",")
    else:
      parts = [opt]
    prods = []
    try:
      for part in parts:
        if part.count("gt"):
          self.minprod = int(part.replace("gt", ""))
          continue
        if part.count("-"):
          prods.extend(list(range(int(part.split("-")[0]), int(part.split("-")[1]) + 1)))
        else:
          prods.append(int(part))
    except ValueError:
      return S_ERROR('ProductionID: bad syntax')
    self.prod = prods

    return S_OK()

  def setFullDetail(self, _opt):
    """Get every individual file's properties, makes this very very slow."""
    self.full_det = True
    return S_OK()

  def setSampleSize(self, opt):
    """Set the number of generator files to sample for cross-section and luminosity."""
    try:
      self.sampleSize = int(opt)
    except ValueError:
      return S_ERROR('SampleSize: bad value')
    return S_OK()

  def setProdTypes(self, opt):
    """Set the prod types to consider."""
    self.ptypes = opt.split(",")
    return S_OK()

  def setStatuses(self, opt):
    """Set the prod statuses."""
    self.statuses = opt.split(",")
    return S_OK()

  def registerSwitches(self):
    """Register all CLI switches."""
    Script.registerSwitch(
        "P:",
        "prods=",
        "Productions: greater than with gt1234, range with 32-56, list with 34,56",
        self.setProdID)
    Script.registerSwitch("p", "precise_detail", "Precise detail, slow", self.setFullDetail)
    Script.registerSwitch("N:", "sample_size=", "The number of generator files to sample for cross-section and "
                          "luminosity, default 100", self.setSampleSize)
    Script.registerSwitch("t:", "types=", "Production Types, comma separated, default all", self.setProdTypes)
    Script.registerSwitch("S:", "Statuses=", "Statuses, comma separated, default all", self.setStatuses)
    Script.setUsageMessage('\n'.join([__doc__.split('\n')[1],
                                         '\nUsage:',
                                         '  %s [option|cfgfile] ...\n' % Script.scriptName]))


@DIRACScript()
def main():
  clip = _Params()
  clip.registerSwitches()
  Script.parseCommandLine()
  from ILCDIRAC.Core.Utilities.HTML import Table
  from ILCDIRAC.Core.Utilities.ProcessList import ProcessList
  from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient
  from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
  from DIRAC import gConfig, gLogger
  prod = clip.prod
  full_detail = clip.full_det
  fc = FileCatalogClient()

  processlist = gConfig.getValue('/LocalSite/ProcessListPath', '')
  prl = ProcessList(processlist)
  processesdict = prl.getProcessesDict()

  trc = TransformationClient()
  prodids = []
  if not prod:
    conddict = {}
    conddict['Status'] = clip.statuses
    if clip.ptypes:
      conddict['Type'] = clip.ptypes
    res = trc.getTransformations(conddict)
    if res['OK']:
      for transfs in res['Value']:
        prodids.append(transfs['TransformationID'])
  else:
    prodids.extend(prod)

  metadata = []

  gLogger.info("Will run on prods %s" % str(prodids))

  for prodID in prodids:
    if prodID < clip.minprod:
      continue
    meta = {}
    meta['ProdID'] = prodID
    res = trc.getTransformation(str(prodID))
    if not res['OK']:
      gLogger.error("Error getting transformation %s" % prodID)
      continue
    prodtype = res['Value']['Type']
    proddetail = res['Value']['Description']
    meta['Datatype'] = 'DST' if prodtype in ('MCReconstruction', 'MCReconstruction_Overlay') \
                       else 'gen' if prodtype == 'MCGeneration' \
                       else 'SIM' if prodtype == 'MCSimulation' else None
    if prodtype in ['Split', 'Merge']:
      gLogger.warn("Invalid query for %s productions" % prodtype)
      continue
    if not meta['Datatype']:
      gLogger.error("Unknown production type %s" % prodtype)
      continue
    gLogger.verbose('Looking for files: %s' % meta)
    res = fc.findFilesByMetadata(meta)
    if not res['OK']:
      gLogger.error(res['Message'])
      continue
    lfns = res['Value']
    nb_files = len(lfns)
    if not lfns:
      gLogger.warn("No files found for prod %s" % prodID)
      continue
    path = os.path.dirname(lfns[0])
    gLogger.verbose('Looking for user metadata')
    res = fc.getDirectoryUserMetadata(path)
    if not res['OK']:
      gLogger.warn('No meta data found for %s' % path)
      continue
    dirmeta = {'proddetail': proddetail,
               'prodtype': prodtype,
               'nb_files': nb_files,
               }
    dirmeta.update(res['Value'])
    lumi = 0.
    nbevts = 0
    files = 0
    xsec = 0.0
    if not full_detail:
      info = _getFileInfo(lfns[0], fc)
      lumi = info[0] * len(lfns)
      nbevts = info[1] * len(lfns)
      if info[2]:
        xsec += info[2]
        files += 1
    else:
      for lfn in lfns:
        info = _getFileInfo(lfn, fc)
        lumi += info[0]
        nbevts += info[1]
        if info[2]:
          xsec += info[2]
          files += 1
        gLogger.verbose('Found lumi: %s, xsec: %s' % (lumi, xsec))
        if not lumi:  # we are guessing none of the files have that information at this point
          gLogger.notice('Files do not have xsection info, looking at ancestors now')
          nbevts = info[1] * len(lfns)
          break

    if not lumi:
      xsec = 0
      files = 0
      depthDict = defaultdict(set)
      gLogger.verbose('Looking for Ancestors of %s files' % len(lfns))
      for index, lfnChunk in enumerate(breakListIntoChunks(lfns[:1000], 1000)):
        gLogger.verbose('Looking at chunk %s/%s' % (index, len(lfns) // 1000))
        res = fc.getFileAncestors(lfnChunk, [1, 2, 3, 4])
        if not res['OK']:
          gLogger.error('failed to find ancenstors')
          return res
        for lfn, ancestorsDict in res['Value']['Successful'].items():
          for ancestor, dep in ancestorsDict.items():
            depthDict[dep].add(ancestor)
      gLogger.verbose('Found all the ancestors')
      oldestAncestorID = sorted(depthDict)[-1]
      oldestAncestors = depthDict[oldestAncestorID]
      numberOfAncestors = len(oldestAncestors)
      sampleSize = clip.sampleSize if not full_detail else numberOfAncestors
      gLogger.verbose('Sampling %s from %s' % (sampleSize, numberOfAncestors))
      for index, ancestor in enumerate(random.sample(oldestAncestors, sampleSize)):
        gLogger.verbose('Getting info for ancestor %s / %s' % (index, sampleSize))
        info = _getFileInfo(ancestor, fc)
        # lumi is not used later on
        lumi += info[0] * float(numberOfAncestors) / float(sampleSize)
        if info[2]:
          xsec += info[2]
          files += 1
        gLogger.verbose('Found %s, %s, %s' % (info[0], info[2], files))

    dirmeta['CrossSection'] = xsec / float(files) if xsec and files else 0.0
    dirmeta['NumberOfEvents'] = nbevts if nbevts else 0

    dirmeta['detail'] = ''
    if dirmeta['EvtType'] in processesdict:
      if 'Detail' in processesdict[dirmeta['EvtType']]:
        detail = processesdict[dirmeta['EvtType']]['Detail']
    else:
      detail = dirmeta['EvtType']

    dirmeta['MomProdID'] = 0
    if prodtype != 'MCGeneration':
      res = trc.getTransformationMetaDataQuery(str(prodID), 'Input')
      if res['OK']:
        if 'ProdID' in res['Value']:
          dirmeta['MomProdID'] = res['Value']['ProdID']

    dirmeta['detail'] = _translate(detail)
    metadata.append(dirmeta)

  detectors = {'ILD': {'SIM': [], 'REC': [], 'title': 'ILD CDR'},
               'SID': {'SIM': [], 'REC': [], 'title': 'SID CDR'},
               'sid': {'SIM': [], 'REC': [], 'title': 'sid dbd prod'},
               'FCCee_o1_v04': {'SIM': [], 'REC': [], 'title': 'FCC studies'},
               'gen': [],
               }

  corres = {'MCGeneration': 'gen',
            'MCSimulation': 'SIM',
            'MCReconstruction': 'REC',
            'MCReconstruction_Overlay': 'REC'}

  for channel in metadata:
    if 'DetectorType' not in channel:
      detectors['gen'].append((channel['detail'],
                               channel['Energy'],
                               channel['ProdID'],
                               channel['nb_files'],
                               channel['NumberOfEvents'] / channel['nb_files'],
                               channel['NumberOfEvents'],
                               channel['CrossSection'], str(channel['proddetail'])))
    else:
      if channel['DetectorType'] not in detectors:
        gLogger.error("This is unknown detector", channel['DetectorType'])
        continue
      detectors[channel['DetectorType']][corres[channel['prodtype']]].append((channel['detail'],
                                                                              channel['Energy'],
                                                                              channel['DetectorType'],
                                                                              channel['ProdID'],
                                                                              channel['nb_files'],
                                                                              channel['NumberOfEvents'] / channel['nb_files'],
                                                                              channel['NumberOfEvents'],
                                                                              channel['CrossSection'],
                                                                              channel['MomProdID'],
                                                                              str(channel['proddetail'])))
  header_row_tuple_gen = ('Channel', 'Energy', 'ProdID', 'Tasks', 'Average Evts/task', 'Statistics',
                          'Cross Section (fb)', 'Comment')

  header_row_tuple_det = ('Channel', 'Energy', 'Detector', 'ProdID', 'Number of Files', 'Events/File',
                          'Statistics', 'Cross Section (fb)', 'Origin ProdID', 'Comment')

  with open('tables.html', 'w') as of:
    of.write(textwrap.dedent("""
                             <!DOCTYPE html>
                             <html>
                               <head>
                                 <title> Production summary </title>
                               </head>
                               <body>
                             """).strip())
    if detectors['gen']:
      of.write("<h1>gen prods</h1>\n")
      table = Table(header_row=header_row_tuple_gen)
      table.rows.extend(detectors['gen'])
      of.write(str(table))
      gLogger.info("Gen prods")
      gLogger.info(str(table))

    detectors.pop('gen')

    for detName, infos in detectors.items():
      if any(infos.get(pt) for pt in ('SIM', 'REC')):
        of.write('<h1>%s prods</h1>\n' % infos['title'])
        for ptype in ('SIM', 'REC'):
          if infos[ptype]:
            of.write('<h2>%s</h2>\n' % ptype)
            table = Table(header_row=header_row_tuple_det)
            table.rows.extend(infos[ptype])
            of.write(str(table))
            gLogger.info('%s prods %s' % (infos['title'], ptype))
            gLogger.info(str(table))

    of.write(textwrap.dedent("""
                               </body>
                             </html>"""))
  gLogger.notice("Check ./tables.html in any browser for the results")
  dexit(0)


if __name__ == "__main__":
  main()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Example on how to submit jobs using parametric input data.

:since: Nov 6, 2013

:author: sposs
"""
from __future__ import absolute_import
from DIRAC.Core.Base import Script
Script.parseCommandLine()

from DIRAC import gLogger, exit as dexit
from DIRAC.Resources.Catalog.FileCatalogClient import FileCatalogClient
from ILCDIRAC.Interfaces.API.DiracILC import DiracILC
from ILCDIRAC.Interfaces.API.NewInterface.Applications import Marlin
from ILCDIRAC.Interfaces.API.NewInterface.UserJob import UserJob

LOG = gLogger.getSubLogger(__name__)

__RCSID__ = "$Id$"


def getFiles():
  """Get the lfns: This is not the point of this example, so keep it out of the main."""
  fc = FileCatalogClient()

  meta = {}
  meta['ProdID'] = 1543
  meta["Datatype"] = "DST"

  result = fc.findFilesByMetadata(meta, "/ilc/prod/clic")
  if not result["OK"]:
    LOG.error(result["Message"])
    dexit(1)
  return result['Value']


def getJob():
  """produce a job: it's always the same, so we don't need to put it in the main."""
  j = UserJob()
  ma = Marlin()
  ma.setVersion("v0111Prod")
  ma.setSteeringFile("clic_ild_cdr_steering.xml")
  ma.setGearFile("clic_ild_cdr.gear")
  result = j.append(ma)
  if not result['OK']:
    LOG.error(result["Message"])
    dexit(1)
  j.setCPUTime(10000)
  j.setOutputSandbox("*.log")
  return j


if __name__ == '__main__':

  lfns = getFiles()  # get a list of files

  d = DiracILC(True, "paramjobtest.rep")  # get your dirac instance

  job = getJob()  # get a job, any can do

  # here is where the interesting stuff happen
  from DIRAC.Core.Utilities.List import breakListIntoChunks
  for flist in breakListIntoChunks(lfns, 200):
    # 200 is the number of files per chunk, and the max number of jobs produced in one go

    # This is the magical line
    job.setParametricInputData(flist)

    # The rest of the sumission is the same
    res = job.submit(d)
    if not res["OK"]:
      LOG.error("Failed to submit the job: ", res["Message"])



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test WasteCPU."""

from __future__ import absolute_import
import unittest
from mock import MagicMock as Mock, patch

from ILCDIRAC.Core.Utilities.WasteCPU import wasteCPUCycles

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Core.Utilities.WasteCPU'


class WasteCPUTest(unittest.TestCase):
  """Test the WasteCPU."""

  def setUp(self):
    pass

  def tearDown(self):
    pass

  def test_success(self):
    """wasteCPUCycles suceeeds to waste............................................................."""
    self.assertTrue(wasteCPUCycles(1)['OK'])

  def test_fail1(self):
    """wasteCPUCycles fails 1......................................................................."""
    with patch("%s.log" % MODULE_NAME, new=Mock(side_effect=ValueError("MockedValue"))):
      self.assertFalse(wasteCPUCycles(1)['OK'])
      self.assertIn("MockedValue", wasteCPUCycles(1)['Message'])

  def test_fail2(self):
    """wasteCPUCycles fails 2......................................................................."""
    with patch("%s.log" % MODULE_NAME, new=Mock(side_effect=RuntimeError("MockedError"))):
      self.assertFalse(wasteCPUCycles(1)['OK'])
      self.assertIn("OtherException", wasteCPUCycles(1)['Message'])
      self.assertIn("RuntimeError('MockedError'", wasteCPUCycles(1)['Message'])


if __name__ == "__main__":
  SUITE = unittest.defaultTestLoader.loadTestsFromTestCase(WasteCPUTest)
  TESTRESULT = unittest.TextTestRunner(verbosity=2).run(SUITE)



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Holding helper objects or functions to create transformations."""
from __future__ import absolute_import
from pprint import pformat

from DIRAC import gLogger

LOG = gLogger.getSubLogger(__name__)


class Task(object):
  """Object holding all information for a task."""

  def __init__(self, metaInput, parameterDict, eventsPerJob,
               metaPrev=None, nbTasks=None, genFile=None, eventsPerBaseFile=None,
               applicationOptions=None, datatype=None, generator=None,
               taskName='',
               ):
    """Initialise task with all the information we need to create a transformation."""
    LOG.notice('Creating task with meta', str(metaInput))
    self.meta = dict(metaInput)
    self.parameterDict = dict(parameterDict)
    self.eventsPerJob = eventsPerJob
    self.metaPrev = dict(metaPrev) if metaPrev else {}
    self.nbTasks = nbTasks
    self.genFile = genFile
    self.eventsPerBaseFile = eventsPerBaseFile
    self.applicationOptions = dict(applicationOptions) if applicationOptions is not None else {}
    self.cliReco = ''
    self.taskName = taskName
    self.sourceName = ''
    self._updateMeta(self.meta)
    self.datatype = datatype
    self.generator = generator

  def __str__(self):
    """Return string representation of Task."""
    return pformat(vars(self))

  def __repr__(self):
    """Return string representation of Task."""
    return pformat(vars(self), width=150, indent=10)

  def _updateMeta(self, metaDict):
    """Ensure the meta dict contains the correct NumberOfEvents."""
    if self.eventsPerJob is not None:
      metaDict['NumberOfEvents'] = self.eventsPerJob
    if self.eventsPerBaseFile:
      metaDict['NumberOfEvents'] = self.eventsPerBaseFile

  def getProdName(self, *args):
    """Create the production name."""
    workflowName = '_'.join([self.parameterDict['process'],
                             self.meta['Energy'],
                             '_'.join(args),
                             self.sourceName,
                             self.taskName,
                             ]).strip('_')
    while '__' in workflowName:
      workflowName = workflowName.replace('__', '_')
    return workflowName



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Resolve SE takes the workflow SE description and returns the list of destination storage elements for uploading an output file."""

from __future__ import absolute_import
__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.SiteSEMapping import getSEsForSite
from DIRAC.Core.Utilities.List import uniqueElements
from DIRAC import S_OK, S_ERROR, gLogger, gConfig

LOG = gLogger.getSubLogger(__name__)


def getDestinationSEList(outputSE, site, outputmode='Any'):
  """Evaluate the output SE list from a workflow and return the concrete list of SEs to upload output data."""
  # Add output SE defined in the job description
  LOG.info('Resolving workflow output SE description: %s' % outputSE)

  # Check if the SE is defined explicitly for the site
  prefix = site.split('.')[0]
  country = site.split('.')[-1]
  # Concrete SE name
  result = gConfig.getOptions('/Resources/StorageElements/' + outputSE)
  if result['OK']:
    LOG.info('Found concrete SE %s' % outputSE)
    return S_OK([outputSE])
  # There is an alias defined for this Site
  alias_se = gConfig.getValue('/Resources/Sites/%s/%s/AssociatedSEs/%s' % (prefix, site, outputSE), [])
  if alias_se:
    LOG.info('Found associated SE for site %s' % (alias_se))
    return S_OK(alias_se)

  localSEs = getSEsForSite(site)['Value']
  LOG.verbose('Local SE list is: %s' % (localSEs))
  groupSEs = gConfig.getValue('/Resources/StorageElementGroups/' + outputSE, [])
  LOG.verbose('Group SE list is: %s' % (groupSEs))
  if not groupSEs:
    return S_ERROR('Failed to resolve SE ' + outputSE)

  if outputmode.lower() == "local":
    for se in localSEs:
      if se in groupSEs:
        LOG.info('Found eligible local SE: %s' % (se))
        return S_OK([se])

    # check if country is already one with associated SEs
    associatedSE = gConfig.getValue('/Resources/Countries/%s/AssociatedSEs/%s' % (country, outputSE), '')
    if associatedSE:
      LOG.info('Found associated SE %s in /Resources/Countries/%s/AssociatedSEs/%s' % (associatedSE, country, outputSE))
      return S_OK([associatedSE])

    # Final check for country associated SE
    count = 0
    assignedCountry = country
    while count < 10:
      LOG.verbose('Loop count = %s' % (count))
      LOG.verbose("/Resources/Countries/%s/AssignedTo" % assignedCountry)
      opt = gConfig.getOption("/Resources/Countries/%s/AssignedTo" % assignedCountry)
      if opt['OK'] and opt['Value']:
        assignedCountry = opt['Value']
        LOG.verbose('/Resources/Countries/%s/AssociatedSEs' % assignedCountry)
        assocCheck = gConfig.getOption('/Resources/Countries/%s/AssociatedSEs' % assignedCountry)
        if assocCheck['OK'] and assocCheck['Value']:
          break
      count += 1

    if not assignedCountry:
      return S_ERROR('Could not determine associated SE list for %s' % country)

    alias_se = gConfig.getValue('/Resources/Countries/%s/AssociatedSEs/%s' % (assignedCountry, outputSE), [])
    if alias_se:
      LOG.info('Found alias SE for site: %s' % alias_se)
      return S_OK(alias_se)
    else:
      LOG.error('Could not establish alias SE for country %s from section: /Resources/Countries/%s/AssociatedSEs/%s' %
                (country, assignedCountry, outputSE))
      return S_ERROR('Failed to resolve SE ' + outputSE)

  # For collective Any and All modes return the whole group

  # Make sure that local SEs are passing first
  newSEList = []
  for se in groupSEs:
    if se in localSEs:
      newSEList.append(se)
  uniqueSEs = uniqueElements(newSEList + groupSEs)
  LOG.verbose('Found unique SEs: %s' % (uniqueSEs))
  return S_OK(uniqueSEs)

#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the LFNPathUtilities."""
from __future__ import absolute_import
__RCSID__ = "$Id$"

import pytest

from ILCDIRAC.Core.Utilities.LFNPathUtilities import joinPathForMetaData, cleanUpLFNPath

LOG_PATH = "/ilc/prod/ilc/mc-dbd/ild/"
JOB_ID = 12


@pytest.mark.parametrize('paths, expectedPath',
                         [(("/ilc", "grid", "softwareVersion", "/"), "/ilc/grid/softwareVersion/"),
                          (("/ilc//grid", "/", "softwareVersion", "/"), "/ilc/grid/softwareVersion/"),
                          (("/ilc//grid", "/", "softwareVersion/", "/"), "/ilc/grid/softwareVersion/"),
                          ])
def test_joinPathForMetaData(paths, expectedPath):
  """Test for joinPathForMetaData."""
  assert joinPathForMetaData(*paths) == expectedPath


@pytest.mark.parametrize('lfn, expectedPath',
                         [('%s/%s' % (LOG_PATH, str(int(JOB_ID) // 1000).zfill(3)), '/ilc/prod/ilc/mc-dbd/ild/000'),
                          ('LFN:/some/path/to/some/where', '/some/path/to/some/where'),
                          ('lFn:/some/path/to/some/where', '/some/path/to/some/where'),
                          ])
def test_lfnCleanup(lfn, expectedPath):
  """Test for cleanUpLFNPath."""
  assert cleanUpLFNPath(lfn) == expectedPath



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Application used by the Calibration system, not for user jobs."""
from __future__ import absolute_import
import types
from ILCDIRAC.Interfaces.API.NewInterface.Applications import Marlin

from DIRAC import S_OK
from DIRAC.Core.Workflow.Parameter import Parameter

__RCSID__ = "$Id$"


class Calibration(Marlin):
  """Application used in the Calibration System.

  .. warn: Not For user jobs
  """

  def __init__(self, paramdict=None):
    """Initialize."""
    self.calibrationID = 0
    self.workerID = 0
    self.baseSteeringFile = None
    super(Calibration, self).__init__(paramdict)
    # Those 5 need to come after default constructor
    self._modulename = 'Calibration'
    self._moduledescription = 'Module to run calibration'
    self.appname = 'marlin'

  def _checkConsistency(self, job=None):

    super(Calibration, self)._checkConsistency(job)

    return S_OK()

  def _applicationModule(self):
    md1 = super(Calibration, self)._applicationModule()

    md1.addParameter(Parameter("calibrationID", '0', "int", "", "", False, False,
                               "calibration ID"))
    md1.addParameter(Parameter("workerID", '0', "int", "", "", False, False,
                               "worker ID"))
    md1.addParameter(Parameter("baseSteeringFile", '', "string", "", "", False, False,
                               "basic steering file for calibration reconstructions"))
    return md1

  def _applicationModuleValues(self, moduleinstance):

    super(Calibration, self)._applicationModuleValues(moduleinstance)

    moduleinstance.setValue("calibrationID", self.calibrationID)
    moduleinstance.setValue("workerID", self.workerID)
    moduleinstance.setValue("baseSteeringFile", self.baseSteeringFile)

  def setCalibrationID(self, calibrationID):
    """Set calibrationID.

    :param int calibrationID: ID of calibration
    """
    self._checkArgs({'calibrationID': int})
    self.calibrationID = calibrationID

  def setWorkerID(self, workerID):
    """Set workerID.

    :param int workerID: ID of worker node
    """
    self._checkArgs({'workerID': int})
    self.workerID = workerID



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Helper functions for calibration system."""

from __future__ import absolute_import
import re
import os
import fnmatch
import tempfile
import pickle
from xml.etree import ElementTree as et

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Utilities.Subprocess import shellCall
from six.moves import range

LOG = gLogger.getSubLogger(__name__)

# pylint: disable=invalid-name,too-many-locals,too-many-statements

def updateSteeringFile(inFileName, outFileName, parametersToSetup, exceptions=None):
  """Read input xml-file, update values given be dictionary and write result to a new file.

  :param str inFileName: name of input xml-file
  :param str outFileName: name of output xml-file
  :param dict parametersToSetup: dict which contains values which have to be updated.
                                 Keys of dictionary are XPath-string.
                                 E.g.: {"processor/[@name='OuterPlanarDigiProcessor']/parameter[@name='IsStrip']": True}

  :returns: S_OK or S_ERROR
  :rtype: dict
  """
  if exceptions is None:
    exceptions = []
  tree = et.parse(inFileName)

  # FIXME redirect log messegage to LOG class?
  LOG.info("Updating following values for %s:" % outFileName)
  for iPar, iVal in parametersToSetup.items():
    skipParameter = False
    for iEx in exceptions:
      if iEx in iPar:
        skipParameter = True
    if skipParameter:
      continue
    iElement = tree.find(iPar)
    if iElement is None or iVal is None:
      errMsg = ("Cannot update parameter in the steering file! Parameter: %s; Value: %s; inFileName: %s;"
                " outFileName: %s" % (iPar, iVal, inFileName, outFileName))
      LOG.error(errMsg)
      return S_ERROR(errMsg)
    else:
      if isinstance(iVal, (float, int)):
        iVal = str(iVal)

      if iElement.text is None:
        if iVal not in iElement.get('value'):
          LOG.info('%s:\t"%s" --> "%s"' % (iPar, iElement.get('value'), iVal))
      else:
        if iVal not in iElement.text:
          LOG.info('%s:\t"%s" --> "%s"' % (iPar, iElement.text, iVal))
      # remove value attribute since we write value to the text field
      if 'value' in iElement.attrib:
        iElement.attrib.pop('value', None)
      iElement.text = iVal

  tree.write(outFileName)
  return S_OK()


def readValueFromSteeringFile(fileName, xPath):
  """Read value of the node from xml-file.

  :param str fileName: name of xml-file to read
  :param str xPath: xParh of the node to read.
                           E.g.: "processor/[@name='OuterPlanarDigiProcessor']/parameter[@name='IsStrip']"

  :returns: str or None
  :rtype: str
  """
  tree = et.parse(fileName)
  iElement = tree.find(xPath)
  if iElement is not None:
    if iElement.text is None:
      return iElement.get('value')
    else:
      return iElement.text
  else:
    return None


def readParameterDict(inFile='DEFAULT_VALUE'):
  """Read parameter dict from file."""
  if inFile == 'DEFAULT_VALUE':
    import ILCDIRAC.CalibrationSystem.Utilities as utilities
    inFile = os.path.join(utilities.__path__[0], 'auxiliaryFiles/parameterListMarlinSteeringFile.txt')
  outList = {}
  with open(inFile, 'r') as f:
    for iLine in f:
      outList[iLine.split('\n')[0]] = None
  return outList


def readParametersFromSteeringFile(inFileName, parameterDict, exceptions=None):
  """Read parameters (provided in the input dict) from file."""
  if exceptions is None:
    exceptions = []
  tree = et.parse(inFileName)

  for iPar in parameterDict.keys():
    skipParameter = False
    for iEx in exceptions:
      if iEx in iPar:
        skipParameter = True
    if skipParameter:
      continue
    iElement = tree.find(str(iPar))
    if iElement is None:
      return S_ERROR("Cannot read parameter from the steering file! Parameter: %s; inFileName: %s" % (iPar, inFileName))
    else:
      if iElement.text is None:
        parameterDict[iPar] = iElement.get('value')
      else:
        parameterDict[iPar] = iElement.text

  return S_OK()


def convert_and_execute(command_list, fileToSource=''):
  """Take a list, cast every entry of said list to string and executes it in a subprocess.

  :param list command_list: List for a subprocess to execute, that may contain castable non-strings
  :param str fileToSource: file which will be sourced before running command
  :returns: S_OK or S_ERROR
  :rtype: dict
  """
  callString = ''
  for iWord in command_list:
    callString += str(iWord)
    callString += ' '
  callString += '\n'

  tmpFile = tempfile.NamedTemporaryFile(delete=False)
  if fileToSource != '':
    tmpFile.write("source %s\n" % fileToSource)
  tmpFile.write(callString)
  tmpFile.close()

  os.chmod(tmpFile.name, 0o755)
  comm = 'sh -c "%s"' % (tmpFile.name)
  res = shellCall(0, comm)
  os.unlink(tmpFile.name)
  return res


def searchFilesWithPattern(dirName, filePattern):
  """Return list of files which satisfy provided pattern."""
  matches = []
  for root, _, filenames in os.walk(dirName):
    for filename in fnmatch.filter(filenames, filePattern):
      matches.append(os.path.join(root, filename))
  return matches


def saveCalibrationRun(calibRun):
  """Dump instance of calibrationRun class to file."""
  fileName = "calib%s/calibRun_bak.pkl" % (calibRun.calibrationID)
  with open(fileName, 'wb') as f:
    pickle.dump(calibRun, f, pickle.HIGHEST_PROTOCOL)


def loadCalibrationRun(calibrationID):
  """Recover instance of calibrationRun class from file."""
  fileName = "calib%s/calibRun_bak.pkl" % (calibrationID)
  if os.path.exists(fileName):
    with open(fileName, 'rb') as f:
      return pickle.load(f)
  else:
    return None


def addPfoAnalysisProcessor(mainSteeringMarlinRecoFile):
  """Add pfoAnalysis processor (needed for calibration) to Marlin steering file."""
  if not os.path.exists(mainSteeringMarlinRecoFile):
    return S_ERROR("cannot find input steering file: %s" % mainSteeringMarlinRecoFile)

  mainTree = et.ElementTree()
  try:
    mainTree.parse(mainSteeringMarlinRecoFile)
  except et.ParseError as e:
    return S_ERROR("cannot parse input steering file: %s. errMsg: %s" % (mainSteeringMarlinRecoFile, e))
  mainRoot = mainTree.getroot()

  # FIXME TODO properly find path to the file
  # this file should only contains PfoAnalysis processor
  import ILCDIRAC.CalibrationSystem.Utilities as utilities
  pfoAnalysisProcessorFile = os.path.join(utilities.__path__[0], 'auxiliaryFiles/pfoAnalysis.xml')
  if not os.path.exists(pfoAnalysisProcessorFile):
    return S_ERROR("cannot find xml file with pfoAnalysis processor")
  tmpTree = et.parse(pfoAnalysisProcessorFile)
  elementToAdd = tmpTree.getroot()

  if 'MyPfoAnalysis' not in (iEl.attrib['name'] for iEl in mainRoot.iter('processor')):
    tmp1 = mainRoot.find('execute')
    c = et.Element("processor", {"name": "MyPfoAnalysis"})
    tmp1.append(c)
    mainRoot.append(elementToAdd)
    #  mainTree.write(mainSteeringMarlinRecoFile)

    root = mainTree.getroot()
    root_str = et.tostring(root)
    # TODO FIXME why write to "test_<fileName>" file???
    #  with open('test_' + mainSteeringMarlinRecoFile, "w") as of:
    with open(mainSteeringMarlinRecoFile, "w") as of:
      of.write(root_str.decode())

  return S_OK()


def addParameterToProcessor(mainSteeringMarlinRecoFile, processorName, parameterDict):
  """Add parameter to processor inside Marlin steering file."""
  if not os.path.exists(mainSteeringMarlinRecoFile):
    return S_ERROR("cannot find input steering file: %s" % mainSteeringMarlinRecoFile)

  mainTree = et.ElementTree()
  try:
    mainTree.parse(mainSteeringMarlinRecoFile)
  except et.ParseError as e:
    return S_ERROR("cannot parse input steering file: %s. errMsg: %s" % (mainSteeringMarlinRecoFile, e))

  if 'name' not in parameterDict:
    return S_ERROR("parameter dict should have key 'name'")

  mainRoot = mainTree.getroot()

  # each processors is mentioned twixe in the steering file. Once in the execute (just name) and once in the body
  # (with name and type tags).
  procElement = [iEl for iEl in mainRoot.iter('processor') if iEl.attrib['name']
                 == processorName and 'type' in iEl.keys()]
  if len(procElement) > 1:
    return S_ERROR('Multiple processors with given names are found: %s' % procElement)
  elif len(procElement) == 0:
    return S_ERROR("Can't find processor with a name %s in the file %s" % (processorName, mainSteeringMarlinRecoFile))
  else:
    procElement = procElement[0]
  for iSubEl in list(procElement):
    if 'name' in iSubEl.attrib.keys():
      if parameterDict['name'] == iSubEl.attrib['name']:
        return S_ERROR("parameter with name %s already exists in the processor %s" % (parameterDict['name'],
                                                                                      processorName))
  procElement.append(et.Element("parameter", parameterDict))

  root_str = et.tostring(mainRoot)
  with open(mainSteeringMarlinRecoFile, "w") as of:
    of.write(root_str.decode())

  return S_OK()


def splitFilesAcrossJobs(inputFiles, nEventsPerFile, nJobs):
  """Regroup inputFiles dict according to number of jobs.

  Output dict will have a format:

  - arguments of dict: [iJob][iFileType]
  - values of dict: tuple (ordered list of files, startFromEventNumber, nEventsToProcess)

  :param inputFiles: Input list of files for the calibration. Dictionary.
  :type inputFiles: `python:dict`
  :param nEventsPerFile: number of events per file
  :type nEventsPerFile: `python:dict`
  :param int nJobs: Number of jobs to run
  :returns: S_OK with 'Value' element being a new regroupped dict or S_ERROR
  :rtype: dict
  """
  tmpDict = {}
  for iKey, iList in inputFiles.items():
    nEventsPerJob = int(len(iList) * nEventsPerFile[iKey] / nJobs)

    newDict = {}
    for i in range(0, nJobs):
      newDict[i] = []
      indexOfFirstEventInJob = nEventsPerJob * i
      indexOfLastEventInJob = nEventsPerJob * (i + 1) - 1
      indexOfFirstFile = int((indexOfFirstEventInJob) / nEventsPerFile[iKey])
      indexOfLastFile = int((indexOfLastEventInJob) / nEventsPerFile[iKey])
      fileListForJob = iList[indexOfFirstFile:indexOfLastFile + 1]
      startFromEventNumber = indexOfFirstEventInJob - indexOfFirstFile * nEventsPerFile[iKey]
      newDict[i] += (fileListForJob, startFromEventNumber, nEventsPerJob)

    tmpDict[iKey] = newDict

  outDict = {}
  for iJob in range(0, nJobs):
    newDict = {}
    for iType in inputFiles.keys():
      newDict[iType] = tmpDict[iType][iJob]
    outDict[iJob] = newDict

  return outDict


def convert_to_int_list(non_int_list):
  """Take a list and converts each entry to an integer, returning this new list.

  :param list non_int_list: List that contains entries that may not be integers but can be cast
  :returns: List that only contains integers.
  :rtype: list
  """
  result = []
  for entry in non_int_list:
    result.append(int(entry))
  return result


def calibration_creation_failed(results):
  """Return whether or not the creation of all calibration jobs was successful.

  :param results: List of S_OK/S_ERROR dicts that were returned by the submission call
  :returns: True if everything was successful, False otherwise
  :rtype: bool
  """
  success = True
  for job_result in results:
    success = success and job_result['OK']
  return not success



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""ROOT class for common functionality of :doc:`RootScript` and :doc:`RootMacro`."""

from __future__ import absolute_import
import types

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Workflow.Parameter import Parameter

from ILCDIRAC.Interfaces.API.NewInterface.LCApplication import LCApplication
import six

LOG = gLogger.getSubLogger(__name__)
__RCSID__ = "$Id"


class _Root(LCApplication):
  """Root principal class.

  Will inherit in :doc:`RootScript` and :doc:`RootMacro` classes, so don't use this (you can't anyway)!
  """

  def __init__(self, paramdict=None):
    self.arguments = ''
    self.script = None
    super(_Root, self).__init__(paramdict)

  def setScript(self, script):
    """Base method, overloaded in :mod:`~ILCDIRAC.Interfaces.API.NewInterface.Applications.RootScript`"""
    LOG.error("Don't use this!")
    return S_ERROR("Not allowed here")

  def setMacro(self, macro):
    """Base method, overloaded in :mod:`~ILCDIRAC.Interfaces.API.NewInterface.Applications.RootMacro`"""
    LOG.error("Don't use this!")
    return S_ERROR("Not allowed here")

  def setArguments(self, args):
    """ Optional: Define the arguments of the script

    :param str args: Arguments to pass to the command line call

    Note for RootMacro string arguments need to be passed as a raw string with the quotation marks escaped.
    E.g.:

    >>> root = RootMacro()
    >>> root.setArguments(r"\\"myString\\"")

    """
    self._checkArgs({'args': (str,)})
    self.arguments = args
    return S_OK()

  def _applicationModule(self):
    m1 = self._createModuleDefinition()
    m1.addParameter(Parameter("arguments", "", "string", "", "", False, False, "Arguments to pass to the script"))
    m1.addParameter(Parameter("script", "", "string", "", "", False, False, "Script to execute"))
    m1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    return m1

  def _applicationModuleValues(self, moduleinstance):
    moduleinstance.setValue('arguments', self.arguments)
    moduleinstance.setValue("script", self.script)
    moduleinstance.setValue('debug', self.debug)

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):
    """Checks that script is set."""
    if not self.script:
      return S_ERROR("Script or macro not defined")
    if not self.version:
      return S_ERROR("You need to specify the Root version")

    # res = self._checkRequiredApp() ##Check that job order is correct
    # if not res['OK']:
    #  return res

    return S_OK()

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()

  def _resolveLinkedStepParameters(self, stepinstance):
    if isinstance(self._linkedidx, six.integer_types):
      self._inputappstep = self._jobsteps[self._linkedidx]
    if self._inputappstep:
      stepinstance.setLink("InputFile", self._inputappstep.getType(), "OutputFile")
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""General ILC platform configuration, based on LHCb version.

Called from CombinedSoftwareInstallation
"""

from __future__ import absolute_import
import logging
import os
import platform
import re
import sys

__RCSID__ = "$Id$"

# pylint: disable=invalid-name

# CMTCONFIG extraction


def isNewStyleBinary(cmtconfig):
  """check if the CMTCONFIG value is new styled."""
  newstyle = False
  if len(cmtconfig.split("-")) > 1:
    newstyle = True
  return newstyle


def isOldStyleBinary(cmtconfig):
  """check if the CMTCONFIG value is new styled."""
  return not isNewStyleBinary(cmtconfig)


def isBinaryDbg(cmtconfig):
  """check if the CMTCONFIG value is a debug one."""
  bindbg = True
  if isNewStyleBinary(cmtconfig):
    if not cmtconfig.endswith("-dbg"):
      bindbg = False
  else:
    if not cmtconfig.endswith("_dbg"):
      bindbg = False
  return bindbg


def isBinaryOpt(cmtconfig):
  """check if the CMTCONFIG value is an optimized one."""
  binopt = True
  if isBinaryDbg(cmtconfig):
    binopt = False
  return binopt


def getBinaryDbg(cmtconfig):
  """convert CMTCONFIG to debug."""
  cmtdbg = cmtconfig
  if not isBinaryDbg(cmtconfig):
    if isNewStyleBinary(cmtconfig):
      if cmtconfig.endswith("-opt"):
        cmtdbg = "-".join(cmtconfig.split("-")[:-1]) + "-dbg"
      else:
        cmtdbg += "-dbg"
    else:
      cmtdbg += "_dbg"
  return cmtdbg


def getBinaryOpt(cmtconfig):
  """convert CMTCONFIG to optimized."""
  cmtopt = cmtconfig
  if isBinaryDbg(cmtconfig):
    if isNewStyleBinary(cmtconfig):
      cmtopt = "-".join(cmtconfig.split("-")[:-1]) + "-opt"
    else:
      cmtopt = "_".join(cmtconfig.split("_")[:-1])
  return cmtopt


def getCompiler(cmtconfig):
  """extract compiler from CMTCONFIG."""
  compdef = None
  if isNewStyleBinary(cmtconfig):
    compdef = cmtconfig.split("-")[2]
  else:
    if not cmtconfig.startswith("win"):
      compdef = cmtconfig.split("_")[2]
    else:
      compdef = cmtconfig.split("_")[1]
  return compdef


def getPlatformType(cmtconfig):
  """extract platform type (slc5, slc4, etc) from CMTCONFIG."""
  platformtype = None
  if isNewStyleBinary(cmtconfig):
    platformtype = cmtconfig.split("-")[1]
  else:
    platformtype = cmtconfig.split("_")[0]
  if platformtype == "sl5":
    platformtype = "slc5"
  if platformtype == "sl4":
    platformtype = "slc4"
  if platformtype == "sl3":
    platformtype = "slc3"
  return platformtype


def getArchitecture(cmtconfig):
  """extract architecture from CMTCONFIG."""
  architecture = None
  if isNewStyleBinary(cmtconfig):
    architecture = cmtconfig.split("-")[0]
    if architecture == "ia32":
      architecture = "i686"
    if architecture == "amd64":
      architecture = "x86_64"
  else:
    archlist = cmtconfig.split("_")
    if not archlist[0].startswith("win"):
      architecture = archlist[1]
      if architecture == "i686":
        architecture = "ia32"
      if architecture == "x86_64":
        architecture = "i686"
  return architecture


def getConfig(architecture, platformtype, compiler, debug=False):
  """returns the config string."""
  cmtconfig = None
  if platformtype.startswith("win"):
    cmtconfig = "_".join([platformtype, compiler])
  else:
    if architecture == "ia32":
      architecture = "i686"
    elif architecture == "amd64":
      architecture = "x86_64"
    cmtconfig = "-".join([architecture, platformtype, compiler, "opt"])
    if platformtype == "slc4" or platformtype == "slc3" or platformtype == "osx105":
      if architecture in arch_runtime_compatiblity["ia32"]:
        architecture = "ia32"
      elif architecture == "x86_64":
        architecture = "amd64"
      cmtconfig = "_".join([platformtype, architecture, compiler])

  if debug:
    cmtconfig = getBinaryDbg(cmtconfig)

  return cmtconfig


def pathBinaryMatch(path, cmtconfig):
  """returns True if the path belong to the cmtconfig distribution.

  :param str path: file/path to be tested
  :param str cmtconfig: target cmtconfig
  """
  selected = False
  log = logging.getLogger()
  if cmtconfig not in binary_list:
    log.error("the value of CMTCONFIG %s is not supported", cmtconfig)
  else:
    match_str = "%s" % cmtconfig
    if isOldStyleBinary(cmtconfig) and isBinaryOpt(cmtconfig):
      match_str = "%s(?!_dbg)" % cmtconfig
    cfg_match = re.compile(match_str)
    if cfg_match.search(path):
      selected = True
  return selected


def pathSharedMatch(path, dummy_cmtconfig=None):
  """select path with are not part of a binary distribution.

  :param str path: file/dir path to be tested
  :param str dummy_cmtconfig: optional parameter to exclude specific files for a given cmtconfig, UNUSED
  """
  selected = True
  for b in binary_list:
    if pathBinaryMatch(path, b):
      selected = False
      break
  return selected


def pathMatch(path, cmtconfig, shared=False):
  """return True if the path belong to the CMTCONFIG."""
  selected = False
  if not shared:
    selected = pathBinaryMatch(path, cmtconfig)
  else:
    selected = pathSharedMatch(path, cmtconfig)
  return selected


def pathFilter(pathlist, cmtconfig, shared=False):
  """filter the paths."""
  return [p for p in pathlist if pathMatch(p, cmtconfig, shared)]


# officially supported binaries
binary_opt_list = ["x86_64-slc5-gcc43-opt"]  # ,"slc4_ia32_gcc34"]
# future possible supported binaries
extra_binary_opt_list = ["x86_64-slc5-gcc34-opt"]

binary_dbg_list = [getBinaryDbg(x) for x in binary_opt_list]
extra_binary_dbg_list = [getBinaryDbg(x) for x in extra_binary_opt_list]

binary_list = binary_opt_list + binary_dbg_list
extra_binary_list = extra_binary_opt_list + extra_binary_dbg_list


# supported shells
supported_shells = ["csh", "sh", "bat"]


# Native platform guessing


linux_release_files = ["/etc/redhat-release",
                        "/etc/system-release",
                        "/etc/SuSE-release",
                        "/etc/issue", "/etc/issue.net"]

linux_flavour_aliases = {"slc": ["Scientific Linux"],
                          "rhel": ["Redhat Enterprise", "Red Hat Enterprise"],
                          "rh": ["Redhat", "Red Hat"],
                          "fc": ["Fedora", "Fedora Core"],
                          "suse": ["SuSE"],
                          "co": ["CentOS"],
                          "deb": ["Debian"],
                          "ub": ["Ubuntu"],
                          "ml": ["Mandriva Linux"]
                        }
lsb_flavour_aliases = {"sl": ["ScientificSL"],
                          "slc": ["ScientificCERNSLC"],
                          "fc": ["Fedora"],
                          "co": ["CentOS"]
                        }

flavor_runtime_compatibility = {"slc6": ["slc6", "slc5"],
                                 "slc5": ["slc5", "slc4"],
                                 "slc4": ["slc4", "slc3"],
                                 "slc3": ["slc3"],
                                 "rh73": ["rh73"],
                                 "win32": ["win32"],
                                 "osx105": ["osx105"],
                                 "osx106": ["osx105", "osx106"]
                               }

arch_runtime_compatiblity = {"x86_64": ["x86_64", "i686", "i586", "i486", "i386"],
                              "ia64": ["ia64", "i686", "i586", "i486", "i386"],
                              "ia32": ["ia32", "i686", "i586", "i486", "i386"],
                              "i686": ["i686", "i586", "i486", "i386"],
                              "i586": ["i586", "i486", "i386"],
                              "i486": ["i486", "i386"],
                              "i386": ["i386"],
                              "ppc": ["ppc"]
                            }

flavor_runtime_equivalence = {"slc6": ["slc6", "co6", "rhel6"],
                               "slc5": ["slc5", "co5", "rhel5", "ub9", "fc12", "fc11", "fc10"],
                               "slc4": ["slc4", "co4", "rhel4", "deb4"],
                               "slc3": ["slc3", "suse90", "suse100"],
                               "rh73": ["rh73", "suse80", "suse81", "suse82", "suse83"],
                               "win32": ["win32"],
                               "osx105": ["osx105"],
                               "osx106": ["osx106"]
                             }

supported_compilers = {"slc6": ["gcc44"],
                        "slc5": ["gcc43"],
                        "slc4": ["gcc34"],
                        "slc3": ["gcc323"],
                        "win32": ["vc71"],
                        "osx104": ["gcc40"],
                        "osx105": ["gcc401"],
                        "osx106": ["gcc42"]
                      }


class NativeMachine(object):
  """Class to identify the local configuration/platform."""

  def __init__(self):
    self._arch = None
    self._ostype = None
    self._machine = None
    self._osflavor = None
    self._osversion = None
    self._compversion = None
    self._compiler = None
    #self._sysinfo = platform.uname()
    self._sysinfo = None
    if hasattr(platform, "uname"):
      self._sysinfo = platform.uname()
    if sys.platform == "win32":
      self._arch = "32"
      self._ostype = "Windows"
      self._machine = "i686"
    elif sys.platform == "win64":
      self._arch = "64"
      self._ostype = "Windows"
      self._machine = "x86_64"
    else:
      self._ostype = self._sysinfo[0]
      self._machine = self._sysinfo[4]
      if self._ostype in ["Linux", "LynxOS", "Darwin"]:
        if self._machine == "x86_64" or self._machine == "ia64":
          self._arch = "64"
        else:
          self._arch = "32"
        if self._ostype == "Darwin" and os.popen("uname -p").read()[:-1] == "powerpc":
          self._arch = "ppc"

  def sysInfo(self):
    """full platform.uname() list."""
    return self._sysinfo

  def arch(self):
    """returns 32 or 64."""
    return self._arch

  def OSType(self):
    """returns Linux, Darwin, Windows."""
    return self._ostype

  def machine(self):
    """returns i386, i486, i686, x86_64, ia64, power mac."""
    return self._machine

  def system(self):
    """return Linux-i386, Windows-x86_64 ..."""
    return "%s-%s" % (self._ostype, self._machine)
  # OS extraction

  def OSFlavour(self, teststring=None):
    """Return the OSFlavour."""
    if not self._osflavor or teststring:
      if self._ostype == "Windows":
        self._osflavor = self._sysinfo[2]
        self._osversion = self._sysinfo[3]
      elif self._ostype == "SunOS":
        self._osflavor = "sun"
        self._osversion = "4.x"
      elif self._ostype == "Darwin":
        verList = [int(v) for v in self._sysinfo[2].split('.')]
        if len(verList) > 2:
          osMajRelease = verList[0] - 4
          osMinRelease = verList[1]
          if osMajRelease == 3:
            self._osflavor = 'Panther'
          elif osMajRelease == 4:
            self._osflavor = 'Tiger'
          elif osMajRelease == 5:
            self._osflavor = 'Leopard'
          elif osMajRelease == 6:
            self._osflavor = 'Snow Leopard'
          self._osversion = "10.%d.%d" % (osMajRelease, osMinRelease)

      elif self._ostype == "Linux":
        for f in linux_release_files:
          if os.path.exists(f):
            cont = " ".join(open(f).readlines())
            break
        if teststring:
          cont = teststring
        found = False
        for f in linux_flavour_aliases:
          if not found:
            for s in linux_flavour_aliases[f]:
              if not found:
                if cont.upper().find(s.upper()) != -1:
                  self._osflavor = linux_flavour_aliases[f][0]
                  found = True
                  break
              else:
                break
          else:
            break
        vmatch = re.compile(r"\ +(\d+(?:\.\d+)*)")
        matches = vmatch.search(cont)
        if matches:
          self._osversion = matches.group(1)

    return self._osflavor

  def OSVersion(self, position=None, teststring=None):
    """returns the OSVersion."""
    if not self._osversion:
      if self._ostype == "Windows":
        self._osversion = self._sysinfo[3]
      elif self._ostype == "SunOS":
        self._osversion = "4.x"
      elif self._ostype == "Darwin":
        verList = [int(v) for v in self._sysinfo[2].split(".")]
        if len(verList) > 2:
          osMajRelease = verList[0] - 4
          osMinRelease = verList[1]
          self._osversion = "10.%d.%d" % (osMajRelease, osMinRelease)
      elif self._ostype == "Linux":
        for f in linux_release_files:
          if os.path.exists(f):
            cont = " ".join(open(f).readlines())
            break
        if teststring:
          cont = teststring
        vmatch = re.compile(r"\ +(\d+(?:\.\d+)*)")
        matches = vmatch.search(cont)
        if matches:
          self._osversion = matches.group(1)

    osver = self._osversion

    # returns at most the number of position specified.
    if position:
      osver = ".".join(self._osversion.split(".")[:position])

    return osver

  def nativeCompilerVersion(self, position=None):
    """returns the local compiler version."""
    if not self._compversion:
      if self._ostype == "Windows":
        self._compversion = "vc71"
      else:
        compstr = " ".join(os.popen("g++ --version").readlines())[:-1]
        vmatch = re.compile(r"\ +(\d+(?:\.\d+)*)")
        matches = vmatch.search(compstr)
        if matches:
          self._compversion = matches.group(1)

    ncv = self._compversion

    if position:
      ncv = ".".join(self._compversion.split(".")[:position])

    return ncv

  def nativeCompiler(self):
    """returns the native compiler brand."""
    if not self._compiler:
      if self._ostype == "Windows":
        self._compiler = self.nativeCompilerVersion()
      else:
        cvers = [int(c) for c in self.nativeCompilerVersion(position=2).split(".")]
        self._compiler = "gcc%d%d" % (cvers[0], cvers[1])
        if cvers[0] == 3 and cvers[1] < 4:
          self._compiler = "gcc%s" % self.nativeCompilerVersion(position=3).replace(".", "")
        if self._ostype == "Darwin" and self.OSVersion(position=2) == "10.5":
          self._compiler = "gcc%s" % self.nativeCompilerVersion(position=3).replace(".", "")
    return self._compiler
  # CMT derived informations

  def CMTArchitecture(self):
    """returns the CMT architecture."""
    arch = "ia32"
    if re.compile(r'i\d86').match(self.machine()):
      arch = "ia32"
    elif re.compile('x86_64').match(self.machine()):
      arch = "amd64"
    elif re.compile('ia64').match(self.machine()):
      arch = "ia64"
    elif re.compile('power mac', re.I).match(self.machine()):
      arch = "ppc"
    elif self.OSType() == "Windows":
      arch = sys.platform
    return arch

  def CMTSystem(self):
    """returns the CMTBIn variable used by CMT itself."""
    cmtsystem = None
    if self.OSType() == "Windows":
      cmtsystem = "VisualC"
    elif self.OSType() == "Darwin":
      cmtsystem = "Darwin-i386"
    else:
      if self.machine() in arch_runtime_compatiblity["i586"]:
        cmtsystem = "%s-i386" % self.OSType()
      else:
        cmtsystem = "%s-%s" % (self.OSType(), self.machine())
    return cmtsystem

  def CMTOSFlavour(self):
    """returns the CMT short name for the OS flavour and version."""
    cmtflavour = None
    if self.OSType() == "Windows":
      cmtflavour = "win%s" % self.arch()
    elif self.OSType() == "SunOS":
      cmtflavour = self.OSFlavour() + self.OSVersion(position=2).replace(".", "")
    elif self.OSType() == "Darwin":
      cmtflavour = "osx%s" % self.OSVersion(position=2).replace(".", "")
    elif self.OSType() == "Linux":
      for f in linux_flavour_aliases:
        if self.OSFlavour() == linux_flavour_aliases[f][0]:
          cmtflavour = f + self.OSVersion(position=1)
          if self.OSFlavour() == "SuSE" or self.OSFlavour() == "Redhat":
            cmtflavour = f + self.OSVersion(position=2)
          if self.OSFlavour() == "SuSE" and int(self.OSVersion(position=1)) > 10:
            cmtflavour = f + self.OSVersion(position=1)
    return cmtflavour

  def CMTOSEquivalentFlavour(self):
    """returns the CMT short name for the OS compatible flavour and version."""
    cmtflavour = None
    for f in flavor_runtime_equivalence:
      if self.CMTOSFlavour() in flavor_runtime_equivalence[f]:
        cmtflavour = f
        break
    return cmtflavour

  def CMTCompatibleConfig(self, debug=False):
    """return the list of compatible CMT configs."""
    compatibles = []
    equiv = self.CMTOSEquivalentFlavour()
    machine = self.machine()
    if equiv in flavor_runtime_compatibility:
      for f in flavor_runtime_compatibility[equiv]:
        for m in arch_runtime_compatiblity[machine]:
          for c in supported_compilers[f]:
            n = getConfig(m, f, c, debug=False)
            if n not in compatibles:
              compatibles.append(n)
            if debug:
              n = getConfig(m, f, c, debug=True)
              if n not in compatibles:
                compatibles.append(n)

    return compatibles

  def CMTSupportedConfig(self, debug=False):
    """returns the list of supported CMT configs among the compatible ones."""
    compatibles = self.CMTCompatibleConfig(debug)
    supported = []
    for c in compatibles:
      if c in binary_list and c not in supported:
        supported.append(c)
    return supported

  def CMTNativeConfig(self, debug=False):
    """Returns the native configuration if possible.

    Guess also the compiler
    on linux platforms
    """
    comp = self.nativeCompiler()
    mach = self.machine()
    osflav = self.CMTOSFlavour()
    natconf = getConfig(architecture=mach, platformtype=osflav,
                        compiler=comp, debug=debug)
    return natconf



#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test KKMC module."""

from __future__ import print_function
from __future__ import absolute_import
import linecache
import unittest
from mock import patch, MagicMock as Mock
from mock import mock_open
from mock import mock as mock_module
from parameterized import parameterized
import six

from DIRAC import gLogger, S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.Applications import KKMC
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.Applications.KKMC'
BUILTIN_NAME = 'builtins' if six.PY3 else '__builtin__'

gLogger.setLevel("DEBUG")
gLogger.showHeaders(True)

# pylint: disable=protected-access


class KKMCTestCase(unittest.TestCase):
  """Base class for the KKMC test cases."""

  @classmethod
  def setUpClass(cls):
    """Load the Application file into the linecache to prevent exceptions when mocking the builtin open."""
    from ILCDIRAC.Interfaces.API.NewInterface import Application
    for fName in [Application.__file__, mock_module.__file__]:
      if fName.endswith(('.pyc', '.pyo')):
        fName = fName[:-1]
      linecache.getlines(fName)

  @classmethod
  def tearDownClass(cls):
    """Remove all entries from linecache because we mock builtin open."""
    linecache.clearcache()

  def setUp(self):
    """set up the objects."""
    self.kkmc = KKMC({})
    self.kkmc._jobtype = 'User'
    self.kkmc._ops = Mock(name='OpsMock')

  def test_setEvtType(self):
    self.assertFalse(self.kkmc._errorDict)
    self.kkmc.setEvtType('Mu')
    self.assertFalse(self.kkmc._errorDict)
    assertEqualsImproved(self.kkmc.eventType, 'Mu', self)

  @parameterized.expand([(15, False, '_checkArgs'),
                         ('Mu', True, 'setEvtType'),
                         ])
  def test_setEvtType_fail(self, evtType, addedToJob, errorMessage):
    self.assertFalse(self.kkmc._errorDict)
    self.kkmc.addedtojob = addedToJob
    self.kkmc.setEvtType(evtType)
    self.assertIn(errorMessage, self.kkmc._errorDict)

  @patch('os.path.isfile', new=Mock(return_value=True))
  @patch('%s.open' % BUILTIN_NAME, mock_open(read_data='configFile content'))
  def test_setConfigFile(self):
    """Test setConfigFile."""
    self.assertFalse(self.kkmc._errorDict)
    self.kkmc.setConfigFile('/some/path/configFile.input')
    self.assertFalse(self.kkmc._errorDict)
    assertEqualsImproved(self.kkmc.kkmcConfigFile, 'configFile content', self)

  def test_checkworkflow_app_missing(self):
    self.kkmc._inputapp = ['some_depdency', 'unavailable_dependency_fail_on_this']
    self.kkmc._jobapps = ['myjobapp_1', 'some_dependency']
    assertDiracFailsWith(self.kkmc._checkWorkflowConsistency(), 'job order not correct', self)

  def test_checkworkflow_empty(self):
    self.kkmc._inputapp = []
    self.kkmc._jobapps = []
    assertDiracSucceeds(self.kkmc._checkWorkflowConsistency(), self)

  def test_userjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.kkmc._userjobmodules(module_mock), self)

  def test_prodjobmodules(self):
    module_mock = Mock()
    assertDiracSucceeds(self.kkmc._prodjobmodules(module_mock), self)

  def test_userjobmodules_fails(self):
    with patch('%s._setUserJobFinalization' % MODULE_NAME, new=Mock(return_value=S_OK('something'))),\
            patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_test_err'))):
      assertDiracFailsWith(self.kkmc._userjobmodules(None),
                           'userjobmodules failed', self)

  def test_prodjobmodules_fails(self):
    with patch('%s._setApplicationModuleAndParameters' % MODULE_NAME, new=Mock(return_value=S_OK('something'))), \
            patch('%s._setOutputComputeDataList' % MODULE_NAME, new=Mock(return_value=S_ERROR('some_other_test_err'))):
      assertDiracFailsWith(self.kkmc._prodjobmodules(None),
                           'prodjobmodules failed', self)

  def test_checkconsistency_configFile(self):
    self.kkmc.version = 'LCG_97a_FCC_4'
    self.kkmc.kkmcConfigFile = 'kkmc_steer.input'
    assertDiracSucceeds(self.kkmc._checkConsistency(Mock()), self)

  def test_checkconsistency_parameters(self):
    self.kkmc.version = 'LCG_97a_FCC_4'
    self.kkmc.kkmcConfigFile = None
    self.kkmc.eventType = 'Mu'
    self.kkmc.energy = '91.2'
    self.kkmc.numberOfEvents = '1000'
    self.kkmc.outputFile = 'kkmu_1000.LHE'
    assertDiracSucceeds(self.kkmc._checkConsistency(Mock()), self)

  def test_checkconsistency_noVersion(self):
    self.kkmc.version = None
    assertDiracFailsWith(self.kkmc._checkConsistency(Mock()), 'No version found!', self)

  def test_checkconsistency_noConfigFile(self):
    self.kkmc.version = 'LCG_97a_FCC_4'
    self.kkmc.kkmcConfigFile = None
    assertDiracFailsWith(self.kkmc._checkConsistency(Mock()), 'No config file set!', self)

  def test_checkconsistency_noEventType(self):
    self.kkmc.version = 'LCG_97a_FCC_4'
    self.kkmc.kkmcConfigFile = None
    self.kkmc.eventType = None
    self.kkmc.energy = 91.2
    self.kkmc.numberOfEvents = 1000
    self.kkmc.outputFile = 'kkmu_1000.LHE'
    assertDiracFailsWith(self.kkmc._checkConsistency(Mock()), 'No event type set!', self)

  def test_checkconsistency_noEnergy(self):
    self.kkmc.version = 'LCG_97a_FCC_4'
    self.kkmc.kkmcConfigFile = None
    self.kkmc.eventType = 'Mu'
    self.kkmc.energy = None
    self.kkmc.numberOfEvents = 1000
    self.kkmc.outputFile = 'kkmu_1000.LHE'
    assertDiracFailsWith(self.kkmc._checkConsistency(Mock()), 'No energy set!', self)

  def test_checkconsistency_noNumberOfEvents(self):
    self.kkmc.version = 'LCG_97a_FCC_4'
    self.kkmc.kkmcConfigFile = None
    self.kkmc.eventType = 'Mu'
    self.kkmc.energy = 91.2
    self.kkmc.numberOfEvents = None
    self.kkmc.outputFile = 'kkmu_1000.LHE'
    assertDiracFailsWith(self.kkmc._checkConsistency(Mock()), 'No number of events set!', self)

  def test_checkconsistency_noOutputFile(self):
    self.kkmc.version = 'LCG_97a_FCC_4'
    self.kkmc.kkmcConfigFile = None
    self.kkmc.eventType = 'Mu'
    self.kkmc.energy = 91.2
    self.kkmc.numberOfEvents = 1000
    self.kkmc.outputFile = None
    assertDiracFailsWith(self.kkmc._checkConsistency(Mock()), 'No output file set!', self)


def runTests():
  """Runs our tests."""
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(KKMCTestCase)
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)
  print(testResult)


if __name__ == '__main__':
  runTests()
